@@ -46,6 +46,7 @@
 #include "brasero-session.h"
 #include "burn-plugin-manager.h"
 #include "burn-image-format.h"
+#include "libbrasero-marshal.h"
 
 #include "brasero-tags.h"
 #include "brasero-track-image.h"
@@ -54,6 +55,16 @@
 #include "brasero-burn-lib.h"
 #include "brasero-session-helper.h"
 
+/**
+ * SECTION:brasero-session-cfg
+ * @short_description: Configure automatically a #BraseroBurnSession object
+ * @see_also: #BraseroBurn, #BraseroBurnSession
+ * @include: brasero-session-cfg.h
+ *
+ * This object configures automatically a session reacting to any change
+ * made to the various parameters.
+ **/
+
 typedef struct _BraseroSessionCfgPrivate BraseroSessionCfgPrivate;
 struct _BraseroSessionCfgPrivate
 {
@@ -89,9 +100,37 @@ G_DEFINE_TYPE (BraseroSessionCfg, brasero_session_cfg, BRASERO_TYPE_SESSION_SPAN
 #define BRASERO_DRIVE_PROPERTIES_KEY		"/apps/brasero/drives"
 
 /**
- * Manages the output path
+ * This is to handle tags (and more particularly video ones)
  */
 
+static void
+brasero_session_cfg_tag_changed (BraseroBurnSession *session,
+                                 const gchar *tag)
+{
+	if (!strcmp (tag, BRASERO_VCD_TYPE)) {
+		int svcd_type;
+
+		svcd_type = brasero_burn_session_tag_lookup_int (session, BRASERO_VCD_TYPE);
+		if (svcd_type != BRASERO_SVCD)
+			brasero_burn_session_tag_add_int (session,
+			                                  BRASERO_VIDEO_OUTPUT_ASPECT,
+			                                  BRASERO_VIDEO_ASPECT_4_3);
+	}
+}
+
+/**
+ * brasero_session_cfg_has_default_output_path:
+ * @cfg: a #BraseroSessionCfg
+ *
+ * This function returns whether the path returned
+ * by brasero_burn_session_get_output () is an 
+ * automatically created one.
+ *
+ * Return value: a #gboolean. TRUE if the path(s)
+ * creation is handled by @session, FALSE if it was
+ * set.
+ **/
+
 gboolean
 brasero_session_cfg_has_default_output_path (BraseroSessionCfg *session)
 {
@@ -115,7 +154,7 @@ brasero_session_cfg_wrong_extension_signal (BraseroSessionCfg *session)
 	
 	return_value.g_type = 0;
 	g_value_init (&return_value, G_TYPE_BOOLEAN);
-	g_value_set_int (&return_value, FALSE);
+	g_value_set_boolean (&return_value, FALSE);
 
 	g_signal_emitv (instance_and_params,
 			session_cfg_signals [WRONG_EXTENSION_SIGNAL],
@@ -134,6 +173,8 @@ brasero_session_cfg_set_output_image (BraseroBurnSession *session,
 				      const gchar *toc)
 {
 	gchar *dot;
+	gchar *set_toc = NULL;
+	gchar * set_image = NULL;
 	BraseroBurnResult result;
 	BraseroBurnSessionClass *klass;
 	const gchar *suffixes [] = {".iso",
@@ -142,8 +183,39 @@ brasero_session_cfg_set_output_image (BraseroBurnSession *session,
 				    ".toc",
 				    NULL };
 
-	/* First set all information */
+	/* Make sure something actually changed */
 	klass = BRASERO_BURN_SESSION_CLASS (brasero_session_cfg_parent_class);
+	klass->get_output_path (BRASERO_BURN_SESSION (session),
+	                        &set_image,
+	                        &set_toc);
+
+	if (!set_image && !set_toc) {
+		/* see if image and toc set paths differ */
+		brasero_burn_session_get_output (BRASERO_BURN_SESSION (session),
+		                                 &set_image,
+		                                 &set_toc);
+		if (set_image && image && !strcmp (set_image, image)) {
+			/* It's the same default path so no 
+			 * need to carry on and actually set
+			 * the path of image. */
+			image = NULL;
+		}
+
+		if (set_toc && toc && !strcmp (set_toc, toc)) {
+			/* It's the same default path so no 
+			 * need to carry on and actually set
+			 * the path of image. */
+			toc = NULL;
+		}
+	}
+
+	if (set_image)
+		g_free (set_image);
+
+	if (set_toc)
+		g_free (set_toc);
+
+	/* First set all information */
 	result = klass->set_output_image (session,
 					  format,
 					  image,
@@ -160,40 +232,75 @@ brasero_session_cfg_set_output_image (BraseroBurnSession *session,
 
 	if (format & BRASERO_IMAGE_FORMAT_BIN) {
 		dot = g_utf8_strrchr (image, -1, '.');
-		if (!strcmp (suffixes [0], dot)) {
+		if (strcmp (suffixes [0], dot)) {
 			gboolean res;
 
 			res = brasero_session_cfg_wrong_extension_signal (BRASERO_SESSION_CFG (session));
-			if (res)
-				brasero_image_format_fix_path_extension (format, FALSE, image);
+			if (res) {
+				gchar *fixed_path;
+
+				fixed_path = brasero_image_format_fix_path_extension (format, FALSE, image);
+				/* NOTE: call ourselves with the fixed path as this way,
+				 * in case the path is the same as the default one after
+				 * fixing the extension we'll keep on using default path */
+				result = brasero_burn_session_set_image_output_full (session,
+				                                                     format,
+				                                                     fixed_path,
+				                                                     toc);
+				g_free (fixed_path);
+			}
 		}
 	}
 	else {
 		dot = g_utf8_strrchr (toc, -1, '.');
 
 		if (format & BRASERO_IMAGE_FORMAT_CLONE
-		&& !strcmp (suffixes [1], dot)) {
+		&& strcmp (suffixes [1], dot)) {
 			gboolean res;
 
 			res = brasero_session_cfg_wrong_extension_signal (BRASERO_SESSION_CFG (session));
-			if (res)
-				brasero_image_format_fix_path_extension (format, FALSE, toc);
+			if (res) {
+				gchar *fixed_path;
+
+				fixed_path = brasero_image_format_fix_path_extension (format, FALSE, toc);
+				result = brasero_burn_session_set_image_output_full (session,
+				                                                     format,
+				                                                     image,
+				                                                     fixed_path);
+				g_free (fixed_path);
+			}
 		}
 		else if (format & BRASERO_IMAGE_FORMAT_CUE
-		     && !strcmp (suffixes [2], dot)) {
+		     && strcmp (suffixes [2], dot)) {
 			gboolean res;
 
 			res = brasero_session_cfg_wrong_extension_signal (BRASERO_SESSION_CFG (session));
-			if (res)
-				brasero_image_format_fix_path_extension (format, FALSE, toc);
+			if (res) {
+				gchar *fixed_path;
+
+				fixed_path = brasero_image_format_fix_path_extension (format, FALSE, toc);
+				result = brasero_burn_session_set_image_output_full (session,
+				                                                     format,
+				                                                     image,
+				                                                     fixed_path);
+				g_free (fixed_path);
+			}
 		}
 		else if (format & BRASERO_IMAGE_FORMAT_CDRDAO
-		     && !strcmp (suffixes [3], dot)) {
+		     && strcmp (suffixes [3], dot)) {
 			gboolean res;
 
 			res = brasero_session_cfg_wrong_extension_signal (BRASERO_SESSION_CFG (session));
-			if (res)
-				brasero_image_format_fix_path_extension (format, FALSE, toc);
+			if (res) {
+				gchar *fixed_path;
+
+				fixed_path = brasero_image_format_fix_path_extension (format, FALSE, toc);
+				result = brasero_burn_session_set_image_output_full (session,
+				                                                     format,
+				                                                     image,
+				                                                     fixed_path);
+				g_free (fixed_path);
+			}
 		}
 	}
 
@@ -213,8 +320,8 @@ brasero_session_cfg_get_output_path (BraseroBurnSession *session,
 	klass = BRASERO_BURN_SESSION_CLASS (brasero_session_cfg_parent_class);
 
 	result = klass->get_output_path (session,
-					 toc,
-					 image);
+					 image,
+					 toc);
 	if (result == BRASERO_BURN_OK)
 		return result;
 
@@ -338,6 +445,16 @@ brasero_session_cfg_get_gconf_key (BraseroSessionCfg *self,
 	return key;
 }
 
+/**
+ * brasero_session_cfg_get_error:
+ * @cfg: a #BraseroSessionCfg
+ *
+ * This function returns the current status and if
+ * autoconfiguration is/was successful.
+ *
+ * Return value: a #BraseroSessionError.
+ **/
+
 BraseroSessionError
 brasero_session_cfg_get_error (BraseroSessionCfg *self)
 {
@@ -352,6 +469,14 @@ brasero_session_cfg_get_error (BraseroSessionCfg *self)
 	return priv->is_valid;
 }
 
+/**
+ * brasero_session_cfg_disable:
+ * @cfg: a #BraseroSessionCfg
+ *
+ * This function disables autoconfiguration
+ *
+ **/
+
 void
 brasero_session_cfg_disable (BraseroSessionCfg *self)
 {
@@ -361,6 +486,23 @@ brasero_session_cfg_disable (BraseroSessionCfg *self)
 	priv->disabled = TRUE;
 }
 
+/**
+ * brasero_session_cfg_enable:
+ * @cfg: a #BraseroSessionCfg
+ *
+ * This function (re)-enables autoconfiguration
+ *
+ **/
+
+void
+brasero_session_cfg_enable (BraseroSessionCfg *self)
+{
+	BraseroSessionCfgPrivate *priv;
+
+	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+	priv->disabled = FALSE;
+}
+
 static void
 brasero_session_cfg_save_drive_flags (BraseroSessionCfg *self,
 				      BraseroMedium *medium)
@@ -422,81 +564,14 @@ brasero_session_cfg_save_drive_properties (BraseroSessionCfg *self,
 }
 
 static void
-brasero_session_cfg_set_drive_properties_flags (BraseroSessionCfg *self,
-						BraseroBurnFlag flags)
+brasero_session_cfg_set_drive_properties_default_flags (BraseroSessionCfg *self)
 {
 	BraseroMedia media;
-	BraseroBurnFlag flag;
-	BraseroDrive *drive;
-	BraseroMedium *medium;
-	BraseroBurnResult result;
-	BraseroBurnFlag original_flags;
 	BraseroSessionCfgPrivate *priv;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
 
-	drive = brasero_burn_session_get_burner (BRASERO_BURN_SESSION (self));
-	if (!drive)
-		return;
-
-	medium = brasero_drive_get_medium (drive);
-	if (!medium)
-		return;
-
-	media = brasero_medium_get_status (medium);
-
-	/* This prevents signals to be emitted while (re-) adding them one by one */
-	priv->inhibit_flag_sig = TRUE;
-
-	original_flags = brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self));
-	if (original_flags == flags)
-		return;
-
-	brasero_burn_session_set_flags (BRASERO_BURN_SESSION (self), BRASERO_BURN_FLAG_NONE);
-
-	priv->supported = BRASERO_BURN_FLAG_NONE;
-	priv->compulsory = BRASERO_BURN_FLAG_NONE;
-	result = brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
-						      &priv->supported,
-						      &priv->compulsory);
-
-	if (result != BRASERO_BURN_OK) {
-		brasero_burn_session_set_flags (BRASERO_BURN_SESSION (self), original_flags | flags);
-		priv->inhibit_flag_sig = FALSE;
-		return;
-	}
-
-	for (flag = BRASERO_BURN_FLAG_EJECT; flag < BRASERO_BURN_FLAG_LAST; flag <<= 1) {
-		/* see if this flag was originally set */
-		if (!(flags & flag))
-			continue;
-
-		/* Don't set write modes now in this case */
-		if (brasero_burn_session_same_src_dest_drive (BRASERO_BURN_SESSION (self))
-		&& (flag & (BRASERO_BURN_FLAG_DAO|BRASERO_BURN_FLAG_RAW)))
-			continue;
-
-		if (priv->compulsory
-		&& (priv->compulsory & brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self))) != priv->compulsory) {
-			brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self), priv->compulsory);
-
-			priv->supported = BRASERO_BURN_FLAG_NONE;
-			priv->compulsory = BRASERO_BURN_FLAG_NONE;
-			brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
-							     &priv->supported,
-							     &priv->compulsory);
-		}
-
-		if (priv->supported & flag) {
-			brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self), flag);
-
-			priv->supported = BRASERO_BURN_FLAG_NONE;
-			priv->compulsory = BRASERO_BURN_FLAG_NONE;
-			brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
-							     &priv->supported,
-							     &priv->compulsory);
-		}
-	}
+	media = brasero_burn_session_get_dest_media (BRASERO_BURN_SESSION (self));
 
 	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
 	||  BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED)
@@ -532,6 +607,37 @@ brasero_session_cfg_set_drive_properties_flags (BraseroSessionCfg *self,
 						     &priv->compulsory);
 	}
 
+#if 0
+	
+	/* NOTE: Stop setting DAO here except if it
+	 * is declared compulsory (but that's handled
+	 * somewhere else) or if it was explicity set.
+	 * If we set it just  by  default when it's
+	 * supported but not compulsory, MULTI
+	 * becomes not supported anymore.
+	 * For data the only way it'd be really useful
+	 * is if we wanted to fit a selection on the disc.
+	 * The problem here is that we don't know
+	 * what the size of the final image is going
+	 * to be.
+	 * For images there are cases when after 
+	 * writing an image the user may want to
+	 * add some more data to it later. As in the
+	 * case of data the only way this flag would
+	 * be useful would be to help fit the image
+	 * on the disc. But I doubt it would really
+	 * help a lot.
+	 * For audio we need it to write things like
+	 * CD-TEXT but in this case the backend
+	 * will return it as compulsory. */
+
+	/* Another case when this flag is needed is
+	 * for DVD-RW quick blanked as they only
+	 * support DAO. But there again this should
+	 * be covered by the backend that should
+	 * return DAO as compulsory. */
+
+	/* Leave the code as a reminder. */
 	/* When copying with same drive don't set write mode, it'll be set later */
 	if (!brasero_burn_session_same_src_dest_drive (BRASERO_BURN_SESSION (self))
 	&&  !(media & BRASERO_MEDIUM_DVD)) {
@@ -548,9 +654,110 @@ brasero_session_cfg_set_drive_properties_flags (BraseroSessionCfg *self,
 							     &priv->compulsory);
 
 			/* NOTE: after setting DAO, some flags may become
-			 * compulsory like BLANK_BEFORE for CDRW with data */
+			 * compulsory like MULTI. */
 		}
 	}
+#endif
+}
+
+static void
+brasero_session_cfg_set_drive_properties_flags (BraseroSessionCfg *self,
+						BraseroBurnFlag flags)
+{
+	BraseroDrive *drive;
+	BraseroMedia media;
+	BraseroBurnFlag flag;
+	BraseroMedium *medium;
+	BraseroBurnResult result;
+	BraseroBurnFlag original_flags;
+	BraseroSessionCfgPrivate *priv;
+
+	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+
+	original_flags = brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self));
+	BRASERO_BURN_LOG ("Resetting all flags");
+	BRASERO_BURN_LOG_FLAGS (original_flags, "Current are");
+	BRASERO_BURN_LOG_FLAGS (flags, "New should be");
+
+	drive = brasero_burn_session_get_burner (BRASERO_BURN_SESSION (self));
+	if (!drive) {
+		BRASERO_BURN_LOG ("No drive");
+		return;
+	}
+
+	medium = brasero_drive_get_medium (drive);
+	if (!medium) {
+		BRASERO_BURN_LOG ("No medium");
+		return;
+	}
+
+	media = brasero_medium_get_status (medium);
+
+	/* This prevents signals to be emitted while (re-) adding them one by one */
+	priv->inhibit_flag_sig = TRUE;
+
+	brasero_burn_session_set_flags (BRASERO_BURN_SESSION (self), BRASERO_BURN_FLAG_NONE);
+
+	priv->supported = BRASERO_BURN_FLAG_NONE;
+	priv->compulsory = BRASERO_BURN_FLAG_NONE;
+	result = brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
+						      &priv->supported,
+						      &priv->compulsory);
+
+	if (result != BRASERO_BURN_OK) {
+		brasero_burn_session_set_flags (BRASERO_BURN_SESSION (self), original_flags | flags);
+		priv->inhibit_flag_sig = FALSE;
+		return;
+	}
+
+	for (flag = BRASERO_BURN_FLAG_EJECT; flag < BRASERO_BURN_FLAG_LAST; flag <<= 1) {
+		/* see if this flag was originally set */
+		if ((flags & flag) == 0)
+			continue;
+
+		/* Don't set write modes now in this case */
+		if (brasero_burn_session_same_src_dest_drive (BRASERO_BURN_SESSION (self))
+		&& (flag & (BRASERO_BURN_FLAG_DAO|BRASERO_BURN_FLAG_RAW)))
+			continue;
+
+		if (priv->compulsory
+		&& (priv->compulsory & brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self))) != priv->compulsory) {
+			brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self), priv->compulsory);
+
+			priv->supported = BRASERO_BURN_FLAG_NONE;
+			priv->compulsory = BRASERO_BURN_FLAG_NONE;
+			brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
+							     &priv->supported,
+							     &priv->compulsory);
+		}
+
+		if (priv->supported & flag) {
+			brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self), flag);
+
+			priv->supported = BRASERO_BURN_FLAG_NONE;
+			priv->compulsory = BRASERO_BURN_FLAG_NONE;
+			brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
+							     &priv->supported,
+							     &priv->compulsory);
+		}
+	}
+
+	if (original_flags & BRASERO_BURN_FLAG_DAO
+	&&  priv->supported & BRASERO_BURN_FLAG_DAO) {
+		/* Only set DAO if it was explicitely requested */
+		brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self), BRASERO_BURN_FLAG_DAO);
+
+		priv->supported = BRASERO_BURN_FLAG_NONE;
+		priv->compulsory = BRASERO_BURN_FLAG_NONE;
+		brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
+						     &priv->supported,
+						     &priv->compulsory);
+
+		/* NOTE: after setting DAO, some flags may become
+		 * compulsory like MULTI. */
+	}
+
+	brasero_session_cfg_set_drive_properties_default_flags (self);
 
 	/* allow flag changed signal again */
 	priv->inhibit_flag_sig = FALSE;
@@ -599,11 +806,15 @@ brasero_session_cfg_set_drive_properties (BraseroSessionCfg *self)
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
 
+	BRASERO_BURN_LOG ("Restoring session properties");
+
 	/* The next two must work as they were checked earlier */
 	drive = brasero_burn_session_get_burner (BRASERO_BURN_SESSION (self));
 	medium = brasero_drive_get_medium (drive);
-	if (!medium || brasero_medium_get_status (medium) == BRASERO_MEDIUM_NONE)
+	if (!medium || brasero_medium_get_status (medium) == BRASERO_MEDIUM_NONE) {
+		BRASERO_BURN_LOG ("No media");
 		return;
+	}
 
 	/* Update/set the rate */
 	client = gconf_client_get_default ();
@@ -730,14 +941,7 @@ brasero_session_cfg_check_size (BraseroSessionCfg *self)
 		return BRASERO_SESSION_NO_OUTPUT;
 	}
 
-	flags = brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self));
-	if (flags & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND))
-		brasero_medium_get_free_space (medium, NULL, &disc_size);
-	else if (brasero_burn_session_can_blank (BRASERO_BURN_SESSION (self)) == BRASERO_BURN_OK)
-		brasero_medium_get_capacity (medium, NULL, &disc_size);
-	else
-		brasero_medium_get_free_space (medium, NULL, &disc_size);
-
+	disc_size = brasero_burn_session_get_available_medium_space (BRASERO_BURN_SESSION (self));
 	if (disc_size < 0)
 		disc_size = 0;
 
@@ -783,6 +987,7 @@ brasero_session_cfg_check_size (BraseroSessionCfg *self)
 		return BRASERO_SESSION_INSUFFICIENT_SPACE;
 	}
 
+	flags = brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self));
 	if (!(flags & BRASERO_BURN_FLAG_OVERBURN)) {
 		BraseroSessionCfgPrivate *priv;
 
@@ -821,9 +1026,7 @@ brasero_session_cfg_set_tracks_audio_format (BraseroBurnSession *session,
 }
 
 static void
-brasero_session_cfg_update (BraseroSessionCfg *self,
-			    gboolean update,
-			    gboolean check)
+brasero_session_cfg_update (BraseroSessionCfg *self)
 {
 	BraseroTrackType *source = NULL;
 	BraseroSessionCfgPrivate *priv;
@@ -883,8 +1086,8 @@ brasero_session_cfg_update (BraseroSessionCfg *self,
 			       0);
 		return;
 	}
-	/* it can be empty with an empty track */
 
+	/* it can be empty with just an empty track */
 	if (brasero_track_type_get_has_medium (source)
 	&&  brasero_track_type_get_medium_type (source) == BRASERO_MEDIUM_NONE) {
 		brasero_track_type_free (source);
@@ -929,8 +1132,6 @@ brasero_session_cfg_update (BraseroSessionCfg *self,
 		return;
 	}
 
-	/* FIXME: another easy error to catch: AUDIO project with a DVD */
-
 	/* make sure there is an output set */
 	burner = brasero_burn_session_get_burner (BRASERO_BURN_SESSION (self));
 	if (!burner) {
@@ -944,61 +1145,105 @@ brasero_session_cfg_update (BraseroSessionCfg *self,
 	}
 
 	/* Check that current input and output work */
-	if (priv->CD_TEXT_modified) {
-		/* Try to redo what we undid (after all a new plugin could have
-		 * been activated in the mean time ...) and see what happens */
-		brasero_track_type_set_image_format (source,
-						     BRASERO_METADATA_INFO|
-						     brasero_track_type_get_image_format (source));
-		result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self),
-							       source,
-							       FALSE);
-		if (result == BRASERO_BURN_OK) {
-			priv->CD_TEXT_modified = FALSE;
-
-			priv->configuring = TRUE;
-			brasero_session_cfg_set_tracks_audio_format (BRASERO_BURN_SESSION (self),
-								     brasero_track_type_get_stream_format (source));
-			priv->configuring = FALSE;
-		}
-		else {
-			/* No, nothing's changed */
+	if (brasero_track_type_get_has_stream (source)) {
+		if (priv->CD_TEXT_modified) {
+			/* Try to redo what we undid (after all a new plugin could have
+			 * been activated in the mean time ...) and see what happens */
 			brasero_track_type_set_stream_format (source,
-							      (~BRASERO_METADATA_INFO) &
+							      BRASERO_METADATA_INFO|
 							      brasero_track_type_get_stream_format (source));
 			result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self),
 								       source,
 								       FALSE);
-		}
-	}
-	else {
-		/* Don't use flags as they'll be adapted later. */
-		result = brasero_burn_session_can_burn (BRASERO_BURN_SESSION (self),
-							FALSE);
-		if (result != BRASERO_BURN_OK
-		&&  brasero_track_type_get_has_stream (source)
-		&& (brasero_track_type_get_stream_format (source) & BRASERO_METADATA_INFO)) {
-			/* Another special case in case some burning backends 
-			 * don't support CD-TEXT for audio (libburn). If no
-			 * other backend is available remove CD-TEXT option but
-			 * tell user... */
-			brasero_track_type_set_stream_format (source,
-							      (~BRASERO_METADATA_INFO) &
-							      brasero_track_type_get_stream_format (source));
-
-			result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self),
-								       source,
-								       FALSE);
-			BRASERO_BURN_LOG ("Tested support without Metadata information (result %d)", result);
 			if (result == BRASERO_BURN_OK) {
-				priv->CD_TEXT_modified = TRUE;
+				priv->CD_TEXT_modified = FALSE;
 
 				priv->configuring = TRUE;
 				brasero_session_cfg_set_tracks_audio_format (BRASERO_BURN_SESSION (self),
-									     brasero_track_type_get_has_stream (source));
+									     brasero_track_type_get_stream_format (source));
 				priv->configuring = FALSE;
 			}
+			else {
+				/* No, nothing's changed */
+				brasero_track_type_set_stream_format (source,
+								      (~BRASERO_METADATA_INFO) &
+								      brasero_track_type_get_stream_format (source));
+				result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self),
+									       source,
+									       FALSE);
+			}
 		}
+		else {
+			result = brasero_burn_session_can_burn (BRASERO_BURN_SESSION (self), FALSE);
+
+			if (result != BRASERO_BURN_OK
+			&& (brasero_track_type_get_stream_format (source) & BRASERO_METADATA_INFO)) {
+				/* Another special case in case some burning backends 
+				 * don't support CD-TEXT for audio (libburn). If no
+				 * other backend is available remove CD-TEXT option but
+				 * tell user... */
+				brasero_track_type_set_stream_format (source,
+								      (~BRASERO_METADATA_INFO) &
+								      brasero_track_type_get_stream_format (source));
+
+				result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self),
+									       source,
+									       FALSE);
+				BRASERO_BURN_LOG ("Tested support without Metadata information (result %d)", result);
+				if (result == BRASERO_BURN_OK) {
+					priv->CD_TEXT_modified = TRUE;
+
+					priv->configuring = TRUE;
+					brasero_session_cfg_set_tracks_audio_format (BRASERO_BURN_SESSION (self),
+										     brasero_track_type_get_has_stream (source));
+					priv->configuring = FALSE;
+				}
+			}
+		}
+	}
+	else if (brasero_track_type_get_has_medium (source)
+	&&  (brasero_track_type_get_medium_type (source) & BRASERO_MEDIUM_HAS_AUDIO)) {
+		BraseroImageFormat format = BRASERO_IMAGE_FORMAT_NONE;
+
+		/* If we copy an audio disc check the image
+		 * type we're writing to as it may mean that
+		 * CD-TEXT cannot be copied.
+		 * Make sure that the writing backend
+		 * supports writing CD-TEXT?
+		 * no, if a backend reports it supports an
+		 * image type it should be able to burn all
+		 * its data. */
+		if (!brasero_burn_session_is_dest_file (BRASERO_BURN_SESSION (self))) {
+			BraseroTrackType *tmp_type;
+
+			tmp_type = brasero_track_type_new ();
+
+			/* NOTE: this is the same as brasero_burn_session_can_burn () */
+			result = brasero_burn_session_get_tmp_image_type_same_src_dest (BRASERO_BURN_SESSION (self), tmp_type);
+			if (result == BRASERO_BURN_OK)
+				format = brasero_track_type_get_image_format (tmp_type);
+			else
+				format = BRASERO_IMAGE_FORMAT_NONE;
+
+			brasero_track_type_free (tmp_type);
+
+			BRASERO_BURN_LOG ("Temporary image type %i", format);
+		}
+		else {
+			result = brasero_burn_session_can_burn (BRASERO_BURN_SESSION (self),
+			                                        FALSE);
+			format = brasero_burn_session_get_output_format (BRASERO_BURN_SESSION (self));
+		}
+
+		priv->CD_TEXT_modified = FALSE;
+		if (!(format & (BRASERO_IMAGE_FORMAT_CDRDAO|BRASERO_IMAGE_FORMAT_CUE)))
+			priv->CD_TEXT_modified = TRUE;
+	}
+	else {
+		/* Don't use flags as they'll be adapted later. */
+		priv->CD_TEXT_modified = FALSE;
+		result = brasero_burn_session_can_burn (BRASERO_BURN_SESSION (self),
+							FALSE);
 	}
 
 	if (result != BRASERO_BURN_OK) {
@@ -1022,6 +1267,16 @@ brasero_session_cfg_update (BraseroSessionCfg *self,
 		return;
 	}
 
+	/* Special case for video projects */
+	if (brasero_track_type_get_has_stream (source)
+	&& BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (source))) {
+		/* Only set if it was not already set */
+		if (brasero_burn_session_tag_lookup (BRASERO_BURN_SESSION (self), BRASERO_VCD_TYPE, NULL) != BRASERO_BURN_OK)
+			brasero_burn_session_tag_add_int (BRASERO_BURN_SESSION (self),
+							  BRASERO_VCD_TYPE,
+							  BRASERO_SVCD);
+	}
+
 	brasero_track_type_free (source);
 
 	/* Configure flags */
@@ -1033,11 +1288,6 @@ brasero_session_cfg_update (BraseroSessionCfg *self,
 						  BRASERO_BURN_FLAG_DUMMY|
 						  BRASERO_BURN_FLAG_NO_TMP_FILES);
 
-	if (update)
-		brasero_session_cfg_set_drive_properties (self);
-	else if (check)
-		brasero_session_cfg_check_drive_settings (self);
-
 	priv->configuring = FALSE;
 
 	/* Finally check size */
@@ -1061,14 +1311,23 @@ brasero_session_cfg_session_loaded (BraseroTrackDataCfg *track,
 				    gboolean is_loaded,
 				    BraseroSessionCfg *session)
 {
+	BraseroBurnFlag session_flags;
+	BraseroSessionCfgPrivate *priv;
+
+	priv = BRASERO_SESSION_CFG_PRIVATE (session);
+	if (priv->disabled)
+		return;
+
+	session_flags = brasero_burn_session_get_flags (BRASERO_BURN_SESSION (session));
 	if (is_loaded) {
 		/* Set the correct medium and add the flag */
 		brasero_burn_session_set_burner (BRASERO_BURN_SESSION (session),
 						 brasero_medium_get_drive (medium));
 
-		brasero_session_cfg_add_drive_properties_flags (session, BRASERO_BURN_FLAG_MERGE);
+		if ((session_flags & BRASERO_BURN_FLAG_MERGE) == 0)
+			brasero_session_cfg_add_drive_properties_flags (session, BRASERO_BURN_FLAG_MERGE);
 	}
-	else
+	else if ((session_flags & BRASERO_BURN_FLAG_MERGE) != 0)
 		brasero_session_cfg_rm_drive_properties_flags (session, BRASERO_BURN_FLAG_MERGE);
 }
 
@@ -1093,11 +1352,9 @@ brasero_session_cfg_track_added (BraseroBurnSession *session,
 	 * - reload saved flags
 	 * - check if all flags are thereafter supported
 	 * - check available formats for path
-	 * - set one path
-	 */
-	brasero_session_cfg_update (BRASERO_SESSION_CFG (session),
-				    TRUE,
-				    FALSE);
+	 * - set one path */
+	brasero_session_cfg_update (BRASERO_SESSION_CFG (session));
+	brasero_session_cfg_set_drive_properties (BRASERO_SESSION_CFG (session));
 }
 
 static void
@@ -1123,9 +1380,8 @@ brasero_session_cfg_track_removed (BraseroBurnSession *session,
 	 * - check available formats for path
 	 * - set one path
 	 */
-	brasero_session_cfg_update (BRASERO_SESSION_CFG (session),
-				    TRUE,
-				    FALSE);
+	brasero_session_cfg_update (BRASERO_SESSION_CFG (session));
+	brasero_session_cfg_set_drive_properties (BRASERO_SESSION_CFG (session));
 }
 
 static void
@@ -1145,9 +1401,8 @@ brasero_session_cfg_track_changed (BraseroBurnSession *session,
 	 * - check available formats for path
 	 * - set one path
 	 */
-	brasero_session_cfg_update (BRASERO_SESSION_CFG (session),
-				    TRUE,
-				    FALSE);
+	brasero_session_cfg_update (BRASERO_SESSION_CFG (session));
+	brasero_session_cfg_set_drive_properties (BRASERO_SESSION_CFG (session));
 }
 
 static void
@@ -1155,22 +1410,55 @@ brasero_session_cfg_output_changed (BraseroBurnSession *session,
 				    BraseroMedium *former)
 {
 	BraseroSessionCfgPrivate *priv;
+	BraseroTrackType *type;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (session);
 	if (priv->disabled)
 		return;
 
+	/* Case for video project */
+	type = brasero_track_type_new ();
+	brasero_burn_session_get_input_type (session, type);
+
+	if (brasero_track_type_get_has_stream (type)
+	&&  BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (type))) {
+		BraseroMedia media;
+
+		media = brasero_burn_session_get_dest_media (session);
+		if (media & BRASERO_MEDIUM_DVD)
+			brasero_burn_session_tag_add_int (session,
+			                                  BRASERO_DVD_STREAM_FORMAT,
+			                                  BRASERO_AUDIO_FORMAT_AC3);
+		else if (media & BRASERO_MEDIUM_CD)
+			brasero_burn_session_tag_add_int (session,
+							  BRASERO_DVD_STREAM_FORMAT,
+							  BRASERO_AUDIO_FORMAT_MP2);
+		else {
+			BraseroImageFormat format;
+
+			format = brasero_burn_session_get_output_format (session);
+			if (format == BRASERO_IMAGE_FORMAT_CUE)
+				brasero_burn_session_tag_add_int (session,
+								  BRASERO_DVD_STREAM_FORMAT,
+								  BRASERO_AUDIO_FORMAT_MP2);
+			else
+				brasero_burn_session_tag_add_int (session,
+								  BRASERO_DVD_STREAM_FORMAT,
+								  BRASERO_AUDIO_FORMAT_AC3);
+		}
+	}
+
+	brasero_track_type_free (type);
+
 	brasero_session_cfg_save_drive_properties (BRASERO_SESSION_CFG (session),
 						   former);
 
 	/* In this case need to :
 	 * - load flags 
 	 * - check if all flags are thereafter supported
-	 * - for images, set a path if it wasn't already set
-	 */
-	brasero_session_cfg_update (BRASERO_SESSION_CFG (session),
-				    TRUE,
-				    FALSE);
+	 * - for images, set a path if it wasn't already set */
+	brasero_session_cfg_update (BRASERO_SESSION_CFG (session));
+	brasero_session_cfg_set_drive_properties (BRASERO_SESSION_CFG (session));
 }
 
 static void
@@ -1187,9 +1475,8 @@ brasero_session_cfg_caps_changed (BraseroPluginManager *manager,
 	 * - new flags are supported or not supported anymore
 	 * - new image types as input/output are supported
 	 * - if the current set of flags/input/output still works */
-	brasero_session_cfg_update (self,
-				    FALSE,
-				    TRUE);
+	brasero_session_cfg_update (self);
+	brasero_session_cfg_check_drive_settings (self);
 }
 
 static void
@@ -1198,6 +1485,8 @@ brasero_session_cfg_flags_changed (BraseroBurnSession *session)
 	BraseroSessionCfgPrivate *priv;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (session);
+	if (priv->disabled)
+		return;
 
 	/* when we update the flags we don't want a
 	 * whole series of "flags-changed" emitted.
@@ -1206,37 +1495,42 @@ brasero_session_cfg_flags_changed (BraseroBurnSession *session)
 		g_signal_stop_emission_by_name (session, "flags-changed");
 }
 
+/**
+ * brasero_session_cfg_add_flags:
+ * @cfg: a #BraseroSessionCfg
+ * @flags: a #BraseroBurnFlag
+ *
+ * Adds all flags from @flags that are supported.
+ *
+ **/
+
 void
 brasero_session_cfg_add_flags (BraseroSessionCfg *self,
 			       BraseroBurnFlag flags)
 {
 	BraseroSessionCfgPrivate *priv;
-	BraseroDrive *drive;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+
 	if ((priv->supported & flags) != flags)
 		return;
 
 	if ((brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self)) & flags) == flags)
 		return;
 
-	brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self), flags);
-	priv->supported = BRASERO_BURN_FLAG_NONE;
-	priv->compulsory = BRASERO_BURN_FLAG_NONE;
-	brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
-					     &priv->supported,
-					     &priv->compulsory);
-
-	/* Always save flags */
-	drive = brasero_burn_session_get_burner (BRASERO_BURN_SESSION (self));
-	if (drive && brasero_drive_get_medium (drive))
-		brasero_session_cfg_save_drive_flags (self, brasero_drive_get_medium (drive));
-
-	brasero_session_cfg_update (self,
-				    FALSE,
-				    FALSE);
+	brasero_session_cfg_add_drive_properties_flags (self, flags);
+	brasero_session_cfg_update (self);
 }
 
+/**
+ * brasero_session_cfg_remove_flags:
+ * @cfg: a #BraseroSessionCfg
+ * @flags: a #BraseroBurnFlag
+ *
+ * Removes all flags that are not compulsory.
+ *
+ **/
+
 void
 brasero_session_cfg_remove_flags (BraseroSessionCfg *self,
 				  BraseroBurnFlag flags)
@@ -1247,40 +1541,61 @@ brasero_session_cfg_remove_flags (BraseroSessionCfg *self,
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
 
 	brasero_burn_session_remove_flag (BRASERO_BURN_SESSION (self), flags);
-	priv->supported = BRASERO_BURN_FLAG_NONE;
-	priv->compulsory = BRASERO_BURN_FLAG_NONE;
-	brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
-					     &priv->supported,
-					     &priv->compulsory);
+
+	/* For this case reset all flags as some flags could
+	 * be made available after the removal of one flag
+	 * Example: After the removal of MULTI, FAST_BLANK
+	 * becomes available again for DVDRW sequential */
+	brasero_session_cfg_set_drive_properties_default_flags (self);
 
 	/* Always save flags */
 	drive = brasero_burn_session_get_burner (BRASERO_BURN_SESSION (self));
 	if (drive && brasero_drive_get_medium (drive))
 		brasero_session_cfg_save_drive_flags (self, brasero_drive_get_medium (drive));
 
-	brasero_session_cfg_update (self,
-				    FALSE,
-				    FALSE);
+	brasero_session_cfg_update (self);
 }
 
+/**
+ * brasero_session_cfg_is_supported:
+ * @cfg: a #BraseroSessionCfg
+ * @flag: a #BraseroBurnFlag
+ *
+ * Checks whether a particular flag is supported.
+ *
+ * Return value: a #gboolean. TRUE if it is supported;
+ * FALSE otherwise.
+ **/
+
 gboolean
 brasero_session_cfg_is_supported (BraseroSessionCfg *self,
-				  BraseroBurnFlag flags)
+				  BraseroBurnFlag flag)
 {
 	BraseroSessionCfgPrivate *priv;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
-	return (priv->supported & flags) == flags;
+	return (priv->supported & flag) == flag;
 }
 
+/**
+ * brasero_session_cfg_is_compulsory:
+ * @cfg: a #BraseroSessionCfg
+ * @flag: a #BraseroBurnFlag
+ *
+ * Checks whether a particular flag is compulsory.
+ *
+ * Return value: a #gboolean. TRUE if it is compulsory;
+ * FALSE otherwise.
+ **/
+
 gboolean
 brasero_session_cfg_is_compulsory (BraseroSessionCfg *self,
-				   BraseroBurnFlag flags)
+				   BraseroBurnFlag flag)
 {
 	BraseroSessionCfgPrivate *priv;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
-	return (priv->compulsory & flags) == flags;
+	return (priv->compulsory & flag) == flag;
 }
 
 static void
@@ -1350,6 +1665,7 @@ brasero_session_cfg_class_init (BraseroSessionCfgClass *klass)
 	session_class->track_changed = brasero_session_cfg_track_changed;
 	session_class->output_changed = brasero_session_cfg_output_changed;
 	session_class->flags_changed = brasero_session_cfg_flags_changed;
+	session_class->tag_changed = brasero_session_cfg_tag_changed;
 
 	session_cfg_signals [WRONG_EXTENSION_SIGNAL] =
 		g_signal_new ("wrong_extension",
@@ -1357,8 +1673,8 @@ brasero_session_cfg_class_init (BraseroSessionCfgClass *klass)
 		              G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP | G_SIGNAL_ACTION,
 		              0,
 		              NULL, NULL,
-		              g_cclosure_marshal_VOID__VOID,
-		              G_TYPE_NONE,
+		              brasero_marshal_BOOLEAN__VOID,
+		              G_TYPE_BOOLEAN,
 			      0,
 		              G_TYPE_NONE);
 	session_cfg_signals [IS_VALID_SIGNAL] =
@@ -1373,6 +1689,14 @@ brasero_session_cfg_class_init (BraseroSessionCfgClass *klass)
 		              G_TYPE_NONE);
 }
 
+/**
+ * brasero_session_cfg_new:
+ *
+ *  Creates a new #BraseroSessionCfg object.
+ *
+ * Return value: a #BraseroSessionCfg object.
+ **/
+
 BraseroSessionCfg *
 brasero_session_cfg_new (void)
 {
