@@ -55,6 +55,7 @@
 
 static BraseroPluginManager *plugin_manager = NULL;
 static BraseroMediumMonitor *medium_manager = NULL;
+static BraseroBurnCaps *default_caps = NULL;
 
 
 GQuark
@@ -209,13 +210,13 @@ brasero_burn_library_start (int *argc,
 {
 	GConfClient *client;
 
-	BRASERO_BURN_LOG ("Initializing Brasero-%i.%i.%i",
+	BRASERO_BURN_LOG ("Initializing Brasero-burn %i.%i.%i",
 			  BRASERO_MAJOR_VERSION,
 			  BRASERO_MINOR_VERSION,
 			  BRASERO_SUB);
 
 #if defined(HAVE_STRUCT_USCSI_CMD)
-	/* Work around: because on OpenSolaris brasero posiblely be run
+	/* Work around: because on OpenSolaris brasero possibly be run
 	 * as root for a user with 'Primary Administrator' profile,
 	 * a root dbus session will be autospawned at that time.
 	 * This fix is to work around
@@ -250,7 +251,8 @@ brasero_burn_library_start (int *argc,
 		medium_manager = brasero_medium_monitor_get_default ();
 
 	/* initialize plugins */
-	brasero_burn_caps_get_default ();
+	if (!default_caps)
+		default_caps = BRASERO_BURNCAPS (g_object_new (BRASERO_TYPE_BURNCAPS, NULL));
 
 	if (!plugin_manager)
 		plugin_manager = brasero_plugin_manager_get_default ();
@@ -259,6 +261,16 @@ brasero_burn_library_start (int *argc,
 	return TRUE;
 }
 
+BraseroBurnCaps *
+brasero_burn_caps_get_default ()
+{
+	if (!default_caps)
+		g_error ("You must call brasero_burn_library_start () before using API from libbrasero-burn");
+
+	g_object_ref (default_caps);
+	return default_caps;
+}
+
 /**
  * brasero_burn_library_get_plugins_list:
  * 
@@ -290,6 +302,11 @@ brasero_burn_library_stop (void)
 		medium_manager = NULL;
 	}
 
+	if (default_caps) {
+		g_object_unref (default_caps);
+		default_caps = NULL;
+	}
+
 	/* Cleanup the io thing */
 	brasero_io_shutdown ();
 
@@ -357,9 +374,14 @@ brasero_burn_library_input_supported (BraseroTrackType *type)
 }
 
 /**
+ * brasero_burn_library_get_media_capabilities:
+ * @media: a #BraseroMedia
+ *
  * Used to test what the library can do based on the medium type.
  * Returns BRASERO_MEDIUM_WRITABLE if the disc can be written
  * and / or BRASERO_MEDIUM_REWRITABLE if the disc can be erased.
+ *
+ * Returns: a #BraseroMedia
  */
 
 BraseroMedia
