@@ -70,6 +70,9 @@ struct _BraseroDrivePrivate
 	GDrive *gdrive;
 
 	GThread *probe;
+	GMutex *mutex;
+	GCond *cond;
+	GCond *cond_probe;
 	gint probe_id;
 
 	BraseroMedium *medium;
@@ -85,7 +88,11 @@ struct _BraseroDrivePrivate
 	GCancellable *cancel;
 
 	guint probed:1;
+	guint has_medium:1;
 	guint probe_cancelled:1;
+
+	guint locked:1;
+	guint probe_waiting:1;
 };
 
 #define BRASERO_DRIVE_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_DRIVE, BraseroDrivePrivate))
@@ -108,6 +115,9 @@ G_DEFINE_TYPE (BraseroDrive, brasero_drive, G_TYPE_OBJECT);
 
 #define BRASERO_DRIVE_OPEN_ATTEMPTS			5
 
+static void
+brasero_drive_probe_inside (BraseroDrive *drive);
+
 /**
  * brasero_drive_get_gdrive:
  * @drive: a #BraseroDrive
@@ -184,6 +194,52 @@ last_resort:
 	return result;
 }
 
+static void
+brasero_drive_cancel_probing (BraseroDrive *drive)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+
+	priv->probe_waiting = FALSE;
+
+	g_mutex_lock (priv->mutex);
+	if (priv->probe) {
+
+		/* This to signal that we are cancelling */
+		priv->probe_cancelled = TRUE;
+
+		/* This is to wake up the thread if it
+		 * was asleep waiting to retry to get
+		 * hold of a handle to probe the drive */
+		g_cond_signal (priv->cond_probe);
+
+		g_cond_wait (priv->cond, priv->mutex);
+	}
+	g_mutex_unlock (priv->mutex);
+
+	priv->probed = TRUE;
+
+	if (priv->probe_id) {
+		g_source_remove (priv->probe_id);
+		priv->probe_id = 0;
+	}
+
+	if (priv->medium) {
+		BraseroMedium *medium;
+
+		medium = priv->medium;
+		priv->medium = NULL;
+
+		g_signal_emit (drive,
+			       drive_signals [MEDIUM_REMOVED],
+			       0,
+			       medium);
+
+		g_object_unref (medium);
+	}
+}
+
 /**
  * brasero_drive_eject:
  * @drive: #BraseroDrive
@@ -215,13 +271,22 @@ brasero_drive_eject (BraseroDrive *drive,
 		g_cancellable_reset (priv->cancel);
 	}
 
-	BRASERO_MEDIA_LOG ("Trying to eject drive (?)");
-	res = brasero_gio_operation_eject_drive (priv->gdrive,
-						 priv->cancel,
-						 wait,
-						 error);
-	if (res)
-		return TRUE;
+	BRASERO_MEDIA_LOG ("Trying to eject drive");
+	if (priv->gdrive) {
+		/* Cancel any ongoing probing as it
+		 * would prevent the door from being
+		 * opened. */
+		brasero_drive_cancel_probing (drive);
+
+		res = brasero_gio_operation_eject_drive (priv->gdrive,
+							 priv->cancel,
+							 wait,
+							 error);
+		if (res)
+			return TRUE;
+	}
+	else
+		BRASERO_MEDIA_LOG ("No GDrive");
 
 	if (!priv->medium)
 		return FALSE;
@@ -235,6 +300,10 @@ brasero_drive_eject (BraseroDrive *drive,
 	BRASERO_MEDIA_LOG ("Trying to eject volume");
 	gvolume = brasero_volume_get_gvolume (BRASERO_VOLUME (priv->medium));
 	if (gvolume) {
+		/* Cancel any ongoing probing as it
+		 * would prevent the door from being
+		 * opened. */
+		brasero_drive_cancel_probing (drive);
 		res = brasero_gio_operation_eject_volume (gvolume,
 							  priv->cancel,
 							  wait,
@@ -408,11 +477,12 @@ brasero_drive_lock (BraseroDrive *drive,
 		return FALSE;
 
 	result = (brasero_sbc_medium_removal (handle, 1, NULL) == BRASERO_SCSI_OK);
-	if (!result) {
-		BRASERO_MEDIA_LOG ("Device failed to lock");
+	if (result) {
+		BRASERO_MEDIA_LOG ("Device locked");
+		priv->locked = TRUE;
 	}
 	else
-		BRASERO_MEDIA_LOG ("Device locked");
+		BRASERO_MEDIA_LOG ("Device failed to lock");
 
 	brasero_device_handle_close (handle);
 	return result;
@@ -447,13 +517,21 @@ brasero_drive_unlock (BraseroDrive *drive)
 		return FALSE;
 
 	result = (brasero_sbc_medium_removal (handle, 0, NULL) == BRASERO_SCSI_OK);
-	if (!result) {
-		BRASERO_MEDIA_LOG ("Device failed to unlock");
+	if (result) {
+		BRASERO_MEDIA_LOG ("Device unlocked");
+		priv->locked = FALSE;
+
+		if (priv->probe_waiting) {
+			/* See if a probe was waiting */
+			priv->probe_waiting = FALSE;
+			brasero_drive_probe_inside (drive);
+		}
 	}
 	else
-		BRASERO_MEDIA_LOG ("Device unlocked");
+		BRASERO_MEDIA_LOG ("Device failed to unlock");
 
 	brasero_device_handle_close (handle);
+
 	return result;
 }
 
@@ -707,15 +785,6 @@ brasero_drive_can_write (BraseroDrive *drive)
 			      BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL));
 }
 
-static void
-brasero_drive_init (BraseroDrive *object)
-{
-	BraseroDrivePrivate *priv;
-
-	priv = BRASERO_DRIVE_PRIVATE (object);
-	priv->cancel = g_cancellable_new ();
-}
-
 static void
 brasero_drive_medium_probed (BraseroMedium *medium,
 			     BraseroDrive *self)
@@ -727,6 +796,13 @@ brasero_drive_medium_probed (BraseroMedium *medium,
 	/* only when it is probed */
 	/* NOTE: BraseroMedium calls GDK_THREADS_ENTER/LEAVE() around g_signal_emit () */
 	priv->probed = TRUE;
+
+	if (brasero_medium_get_status (priv->medium) == BRASERO_MEDIUM_NONE) {
+		g_object_unref (priv->medium);
+		priv->medium = NULL;
+		return;
+	}
+
 	g_signal_emit (self,
 		       drive_signals [MEDIUM_INSERTED],
 		       0,
@@ -748,6 +824,257 @@ brasero_drive_probing (BraseroDrive *drive)
 	return priv->probed != TRUE;
 }
 
+static gboolean
+brasero_drive_probed_inside (gpointer data)
+{
+	BraseroDrive *self;
+	BraseroDrivePrivate *priv;
+
+	self = BRASERO_DRIVE (data);
+	priv = BRASERO_DRIVE_PRIVATE (self);
+
+	g_mutex_lock (priv->mutex);
+	priv->probe_id = 0;
+	g_mutex_unlock (priv->mutex);
+
+	priv->probed = TRUE;
+
+	if (priv->has_medium) {
+		if (priv->medium) {
+			priv->probed = TRUE;
+			BRASERO_MEDIA_LOG ("Already a medium. Skipping");
+			return FALSE;
+		}
+
+		BRASERO_MEDIA_LOG ("Probing new medium");
+
+		priv->probed = FALSE;
+		priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
+					     "drive", self,
+					     NULL);
+
+		g_signal_connect (priv->medium,
+				  "probed",
+				  G_CALLBACK (brasero_drive_medium_probed),
+				  self);
+	}
+	else if (priv->medium) {
+		BraseroMedium *medium;
+
+		BRASERO_MEDIA_LOG ("Medium removed");
+
+		medium = priv->medium;
+		priv->medium = NULL;
+
+		g_signal_emit (self,
+			       drive_signals [MEDIUM_REMOVED],
+			       0,
+			       medium);
+
+		g_object_unref (medium);
+	}
+
+	return FALSE;
+}
+
+static gpointer
+brasero_drive_probe_inside_thread (gpointer data)
+{
+	gint counter = 0;
+	GTimeVal wait_time;
+	const gchar *device;
+	BraseroScsiErrCode code;
+	BraseroDrivePrivate *priv;
+	BraseroDeviceHandle *handle = NULL;
+	BraseroDrive *drive = BRASERO_DRIVE (data);
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+
+	/* the drive might be busy (a burning is going on) so we don't block
+	 * but we re-try to open it every second */
+	device = brasero_drive_get_device (drive);
+	BRASERO_MEDIA_LOG ("Trying to open device %s", device);
+
+	priv->has_medium = FALSE;
+
+	handle = brasero_device_handle_open (device, FALSE, &code);
+	while (!handle && counter <= BRASERO_DRIVE_OPEN_ATTEMPTS) {
+		sleep (1);
+
+		if (priv->probe_cancelled) {
+			BRASERO_MEDIA_LOG ("Open () cancelled");
+			goto end;
+		}
+
+		counter ++;
+		handle = brasero_device_handle_open (device, FALSE, &code);
+	}
+
+	if (!handle) {
+		BRASERO_MEDIA_LOG ("Open () failed: medium busy");
+		goto end;
+	}
+
+	if (priv->probe_cancelled) {
+		BRASERO_MEDIA_LOG ("Open () cancelled");
+
+		brasero_device_handle_close (handle);
+		goto end;
+	}
+
+	while (brasero_spc1_test_unit_ready (handle, &code) != BRASERO_SCSI_OK) {
+		if (code == BRASERO_SCSI_NO_MEDIUM) {
+			BRASERO_MEDIA_LOG ("No medium inserted");
+
+			brasero_device_handle_close (handle);
+			goto end;
+		}
+
+		if (code != BRASERO_SCSI_NOT_READY) {
+			BRASERO_MEDIA_LOG ("Device does not respond");
+
+			brasero_device_handle_close (handle);
+			goto end;
+		}
+
+		g_get_current_time (&wait_time);
+		g_time_val_add (&wait_time, 2000000);
+
+		g_mutex_lock (priv->mutex);
+		g_cond_timed_wait (priv->cond_probe,
+		                   priv->mutex,
+		                   &wait_time);
+		g_mutex_unlock (priv->mutex);
+
+		if (priv->probe_cancelled) {
+			BRASERO_MEDIA_LOG ("Device probing cancelled");
+
+			brasero_device_handle_close (handle);
+			goto end;
+		}
+	}
+
+	BRASERO_MEDIA_LOG ("Medium inserted");
+	brasero_device_handle_close (handle);
+
+	priv->has_medium = TRUE;
+
+end:
+
+	g_mutex_lock (priv->mutex);
+
+	if (!priv->probe_cancelled)
+		priv->probe_id = g_idle_add (brasero_drive_probed_inside, drive);
+
+	priv->probe = NULL;
+	g_cond_broadcast (priv->cond);
+	g_mutex_unlock (priv->mutex);
+
+	g_thread_exit (0);
+
+	return NULL;
+}
+
+static void
+brasero_drive_probe_inside (BraseroDrive *drive)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+
+	priv->probed = FALSE;
+
+	/* Check that a probe is not already being performed */
+	g_mutex_lock (priv->mutex);
+	if (!priv->probe) {
+		BRASERO_MEDIA_LOG ("Setting new probe");
+
+		if (priv->probe_id) {
+			/* Remove the result reporting as
+			 * the status seem to have changed */
+			g_source_remove (priv->probe_id);
+			priv->probe_id = 0;
+		}
+
+		priv->probe = g_thread_create (brasero_drive_probe_inside_thread,
+					       drive,
+					       FALSE,
+					       NULL);
+	}
+	else
+		BRASERO_MEDIA_LOG ("Ongoing probe");
+
+	g_mutex_unlock (priv->mutex);
+}
+
+static void
+brasero_drive_medium_gdrive_changed_cb (BraseroDrive *gdrive,
+					BraseroDrive *drive)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+	if (priv->locked) {
+		BRASERO_MEDIA_LOG ("Waiting for next unlocking of the drive to probe");
+
+		/* Since the drive was locked, it should
+		 * not be possible that the medium
+		 * actually changed.
+		 * This allows to avoid probing while
+		 * we are burning something.
+		 * Delay the probe until brasero_drive_unlock ()
+		 * is called.  */
+		priv->probe_waiting = TRUE;
+		return;
+	}
+
+	brasero_drive_probe_inside (drive);
+}
+
+static void
+brasero_drive_update_gdrive (BraseroDrive *drive,
+                             GDrive *gdrive)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+	if (priv->gdrive) {
+		g_signal_handlers_disconnect_by_func (priv->gdrive,
+						      brasero_drive_medium_gdrive_changed_cb,
+						      drive);
+		g_object_unref (priv->gdrive);
+	}
+
+	BRASERO_MEDIA_LOG ("Setting GDrive %p", gdrive);
+
+	if (gdrive) {
+		priv->gdrive = g_object_ref (gdrive);
+
+		/* If it's not a fake drive then connect to signal for any
+		 * change and check medium inside */
+		g_signal_connect (priv->gdrive,
+				  "changed",
+				  G_CALLBACK (brasero_drive_medium_gdrive_changed_cb),
+				  drive);
+	}
+
+	if (priv->locked) {
+		BRASERO_MEDIA_LOG ("Waiting for next unlocking of the drive to probe");
+
+		/* Since the drive was locked, it should
+		 * not be possible that the medium
+		 * actually changed.
+		 * This allows to avoid probing while
+		 * we are burning something.
+		 * Delay the probe until brasero_drive_unlock ()
+		 * is called.  */
+		priv->probe_waiting = TRUE;
+		return;
+	}
+
+	brasero_drive_probe_inside (drive);
+}
+
 /**
  * brasero_drive_reprobe:
  * @drive: a #BraseroDrive
@@ -775,97 +1102,22 @@ brasero_drive_reprobe (BraseroDrive *drive)
 		g_drive_poll_for_media (priv->gdrive, NULL, NULL, NULL);
 	}
 
-	if (!priv->medium)
-		return;
+	priv->probe_waiting = FALSE;
 
 	BRASERO_MEDIA_LOG ("Reprobing inserted medium");
-
-	/* remove current medium */
-	medium = priv->medium;
-	priv->medium = NULL;
-
-	g_signal_emit (drive,
-		       drive_signals [MEDIUM_REMOVED],
-		       0,
-		       medium);
-	g_object_unref (medium);
-	priv->probed = FALSE;
-
-	/* try to get a new one */
-	priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
-				     "drive", drive,
-				     NULL);
-	g_signal_connect (priv->medium,
-			  "probed",
-			  G_CALLBACK (brasero_drive_medium_probed),
-			  drive);
-}
-
-static void
-brasero_drive_check_medium_inside_gdrive (BraseroDrive *self)
-{
-	BraseroDrivePrivate *priv;
-
-	priv = BRASERO_DRIVE_PRIVATE (self);
-
-	BRASERO_MEDIA_LOG ("Contents changed %i", g_drive_has_media (priv->gdrive));
-
-	if (g_drive_has_media (priv->gdrive)) {
-		if (priv->medium)
-			return;
-
-		BRASERO_MEDIA_LOG ("Medium inserted");
-
-		priv->probed = FALSE;
-		priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
-					     "drive", self,
-					     NULL);
-
-		g_signal_connect (priv->medium,
-				  "probed",
-				  G_CALLBACK (brasero_drive_medium_probed),
-				  self);
-	}
-	else if (priv->medium) {
-		BraseroMedium *medium;
-
-		BRASERO_MEDIA_LOG ("Medium removed");
-
+	if (priv->medium) {
+		/* remove current medium */
 		medium = priv->medium;
 		priv->medium = NULL;
 
-		g_signal_emit (self,
+		g_signal_emit (drive,
 			       drive_signals [MEDIUM_REMOVED],
 			       0,
 			       medium);
 		g_object_unref (medium);
-		priv->probed = TRUE;
 	}
-	else
-		priv->probed = TRUE;
-}
 
-static void
-brasero_drive_medium_gdrive_changed_cb (BraseroDrive *gdrive,
-					BraseroDrive *drive)
-{
-	brasero_drive_check_medium_inside_gdrive (drive);
-}
-
-static void
-brasero_drive_update_gdrive (BraseroDrive *drive)
-{
-	BraseroDrivePrivate *priv;
-
-	priv = BRASERO_DRIVE_PRIVATE (drive);
-	/* If it's not a fake drive then connect to signal for any
-	 * change and check medium inside */
-	g_signal_connect (priv->gdrive,
-			  "changed",
-			  G_CALLBACK (brasero_drive_medium_gdrive_changed_cb),
-			  drive);
-
-	brasero_drive_check_medium_inside_gdrive (drive);
+	brasero_drive_probe_inside (drive);
 }
 
 static gboolean
@@ -875,9 +1127,11 @@ brasero_drive_probed (gpointer data)
 
 	priv = BRASERO_DRIVE_PRIVATE (data);
 
-	g_thread_join (priv->probe);
-	priv->probe = NULL;
+	g_mutex_lock (priv->mutex);
 	priv->probe_id = 0;
+	g_mutex_unlock (priv->mutex);
+
+	brasero_drive_probe_inside (BRASERO_DRIVE (data));
 
 	return FALSE;
 }
@@ -1009,7 +1263,10 @@ static gpointer
 brasero_drive_probe_thread (gpointer data)
 {
 	gint counter = 0;
+	GTimeVal wait_time;
 	const gchar *device;
+	BraseroScsiResult res;
+	BraseroScsiInquiry hdr;
 	BraseroScsiErrCode code;
 	BraseroDrivePrivate *priv;
 	BraseroDeviceHandle *handle;
@@ -1028,8 +1285,7 @@ brasero_drive_probe_thread (gpointer data)
 
 		if (priv->probe_cancelled) {
 			BRASERO_MEDIA_LOG ("Open () cancelled");
-			priv->probe = NULL;
-			return NULL;
+			goto end;
 		}
 
 		counter ++;
@@ -1038,56 +1294,93 @@ brasero_drive_probe_thread (gpointer data)
 
 	if (priv->probe_cancelled) {
 		BRASERO_MEDIA_LOG ("Open () cancelled");
-		priv->probe = NULL;
-		return NULL;
+		goto end;
 	}
 
-	if (handle) {
-		BraseroScsiInquiry hdr;
-		BraseroScsiResult res;
+	if (!handle) {
+		BRASERO_MEDIA_LOG ("Open () failed: medium busy");
+		goto end;
+	}
 
-		BRASERO_MEDIA_LOG ("Open () succeeded");
-
-		/* get additional information like the name */
-		res = brasero_spc1_inquiry (handle, &hdr, NULL);
-		if (res == BRASERO_SCSI_OK) {
-			gchar *name_utf8;
-			gchar *vendor;
-			gchar *model;
-			gchar *name;
-
-			vendor = g_strndup ((gchar *) hdr.vendor, sizeof (hdr.vendor));
-			model = g_strndup ((gchar *) hdr.name, sizeof (hdr.name));
-			name = g_strdup_printf ("%s %s", g_strstrip (vendor), g_strstrip (model));
-			g_free (vendor);
-			g_free (model);
-
-			/* make sure that's proper UTF-8 */
-			name_utf8 = g_convert_with_fallback (name,
-			                                     -1,
-			                                     "ASCII",
-			                                     "UTF-8",
-			                                     "_",
-			                                     NULL,
-			                                     NULL,
-			                                     NULL);
-			g_free (name);
-
-			priv->name = name_utf8;
+	while (brasero_spc1_test_unit_ready (handle, &code) != BRASERO_SCSI_OK) {
+		if (code == BRASERO_SCSI_NO_MEDIUM) {
+			BRASERO_MEDIA_LOG ("No medium inserted");
+			break;
 		}
 
-		/* Get supported medium types */
-		if (!brasero_drive_get_caps_profiles (drive, handle, &code))
-			brasero_drive_get_caps_2A (drive, handle, &code);
+		if (code != BRASERO_SCSI_NOT_READY) {
+			brasero_device_handle_close (handle);
+			BRASERO_MEDIA_LOG ("Device does not respond");
+			goto end;
+		}
 
-		brasero_device_handle_close (handle);
+		g_get_current_time (&wait_time);
+		g_time_val_add (&wait_time, 2000000);
 
-		BRASERO_MEDIA_LOG ("Drive caps are %d", priv->caps);
+		g_mutex_lock (priv->mutex);
+		g_cond_timed_wait (priv->cond_probe,
+		                   priv->mutex,
+		                   &wait_time);
+		g_mutex_unlock (priv->mutex);
+
+		if (priv->probe_cancelled) {
+			brasero_device_handle_close (handle);
+			BRASERO_MEDIA_LOG ("Device probing cancelled");
+			goto end;
+		}
 	}
-	else
-		BRASERO_MEDIA_LOG ("Open () failed: medium busy");
 
-	priv->probe_id = g_idle_add (brasero_drive_probed, drive);
+	BRASERO_MEDIA_LOG ("Device ready");
+
+	/* get additional information like the name */
+	res = brasero_spc1_inquiry (handle, &hdr, NULL);
+	if (res == BRASERO_SCSI_OK) {
+		gchar *name_utf8;
+		gchar *vendor;
+		gchar *model;
+		gchar *name;
+
+		vendor = g_strndup ((gchar *) hdr.vendor, sizeof (hdr.vendor));
+		model = g_strndup ((gchar *) hdr.name, sizeof (hdr.name));
+		name = g_strdup_printf ("%s %s", g_strstrip (vendor), g_strstrip (model));
+		g_free (vendor);
+		g_free (model);
+
+		/* make sure that's proper UTF-8 */
+		name_utf8 = g_convert_with_fallback (name,
+		                                     -1,
+		                                     "ASCII",
+		                                     "UTF-8",
+		                                     "_",
+		                                     NULL,
+		                                     NULL,
+		                                     NULL);
+		g_free (name);
+
+		priv->name = name_utf8;
+	}
+
+	/* Get supported medium types */
+	if (!brasero_drive_get_caps_profiles (drive, handle, &code))
+		brasero_drive_get_caps_2A (drive, handle, &code);
+
+	brasero_device_handle_close (handle);
+
+	BRASERO_MEDIA_LOG ("Drive caps are %d", priv->caps);
+
+end:
+
+	g_mutex_lock (priv->mutex);
+
+	if (!priv->probe_cancelled)
+		priv->probe_id = g_idle_add (brasero_drive_probed, drive);
+
+	priv->probe = NULL;
+	g_cond_broadcast (priv->cond);
+	g_mutex_unlock (priv->mutex);
+
+	g_thread_exit (0);
+
 	return NULL;
 }
 
@@ -1116,10 +1409,12 @@ brasero_drive_init_real_device (BraseroDrive *drive,
 	 * thread then our whole UI blocks. This medium won't be exported by the
 	 * BraseroDrive that exported until it returns PROBED signal.
 	 * One (good) side effect is that it also improves start time. */
+	g_mutex_lock (priv->mutex);
 	priv->probe = g_thread_create (brasero_drive_probe_thread,
 				       drive,
-				       TRUE,
+				       FALSE,
 				       NULL);
+	g_mutex_unlock (priv->mutex);
 }
 
 static void
@@ -1144,30 +1439,7 @@ brasero_drive_set_property (GObject *object,
 			break;
 
 		gdrive = g_value_get_object (value);
-		if (priv->gdrive) {
-			g_signal_handlers_disconnect_by_func (priv->gdrive,
-							      brasero_drive_medium_gdrive_changed_cb,
-							      object);
-			g_object_unref (priv->gdrive);
-		}
-
-		BRASERO_MEDIA_LOG ("Setting GDrive %p", gdrive);
-
-		if (gdrive) {
-			priv->gdrive = g_object_ref (gdrive);
-			brasero_drive_update_gdrive (BRASERO_DRIVE (object));
-		}
-		else if (!priv->medium) {
-			priv->probed = FALSE;
-			priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
-						     "drive", object,
-						     NULL);
-
-			g_signal_connect (priv->medium,
-					  "probed",
-					  G_CALLBACK (brasero_drive_medium_probed),
-					  object);
-		}
+		brasero_drive_update_gdrive (BRASERO_DRIVE (object), gdrive);
 		break;
 	case PROP_DEVICE:
 		if (!g_value_get_string (value)) {
@@ -1213,6 +1485,19 @@ brasero_drive_get_property (GObject *object,
 	}
 }
 
+static void
+brasero_drive_init (BraseroDrive *object)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (object);
+	priv->cancel = g_cancellable_new ();
+
+	priv->mutex = g_mutex_new ();
+	priv->cond = g_cond_new ();
+	priv->cond_probe = g_cond_new ();
+}
+
 static void
 brasero_drive_finalize (GObject *object)
 {
@@ -1222,15 +1507,21 @@ brasero_drive_finalize (GObject *object)
 
 	BRASERO_MEDIA_LOG ("Finalizing BraseroDrive");
 
-	if (priv->probe) {
-		priv->probe_cancelled = TRUE;
-		g_thread_join (priv->probe);
-		priv->probe = 0;
+	brasero_drive_cancel_probing (BRASERO_DRIVE (object));
+
+	if (priv->mutex) {
+		g_mutex_free (priv->mutex);
+		priv->mutex = NULL;
 	}
 
-	if (priv->probe_id) {
-		g_source_remove (priv->probe_id);
-		priv->probe_id = 0;
+	if (priv->cond) {
+		g_cond_free (priv->cond);
+		priv->cond = NULL;
+	}
+
+	if (priv->cond_probe) {
+		g_cond_free (priv->cond_probe);
+		priv->cond_probe = NULL;
 	}
 
 	if (priv->medium) {
