@@ -435,9 +435,12 @@ brasero_metadata_thumbnail (BraseroMetadata *self)
 
 	/* find the right position and move forward */
 	position = 15 * GST_SECOND;
-	while (position >= priv->info->len)
+	while (position > 0 && position >= priv->info->len)
 		position -= 5 * GST_SECOND;
 
+	if (position <= 0)
+		return FALSE;
+
 	gst_element_set_state (priv->pipeline, GST_STATE_PAUSED);
 
 	priv->snapshot_started = 1;
@@ -645,7 +648,8 @@ brasero_metadata_success (BraseroMetadata *self)
 	}
 
 	/* before leaving, check if we need a snapshot */
-	if (priv->snapshot
+	if (priv->info->len > 0
+	&&  priv->snapshot
 	&&  priv->video_linked
 	&& !priv->snapshot_started)
 		return brasero_metadata_thumbnail (self);
@@ -1024,7 +1028,6 @@ brasero_metadata_install_missing_plugins (BraseroMetadata *self)
 		download = brasero_metadata_is_downloading (detail);
 		if (download) {
 			download->objects = g_slist_prepend (download->objects, self);
-			priv->downloads = g_slist_prepend (priv->downloads, download);
 			g_free (detail);
 			continue;
 		}
@@ -1430,7 +1433,6 @@ brasero_metadata_new_decoded_pad_cb (GstElement *decode,
 	name = gst_structure_get_name (structure);
 	has_audio = (g_strrstr (name, "audio") != NULL);
 	has_video = (g_strrstr (name, "video") != NULL);
-
 	priv->info->has_audio |= has_audio;
 	priv->info->has_video |= has_video;
 
