@@ -145,6 +145,7 @@ error_loading:
 static gboolean
 brasero_dvdcss_thread_finished (gpointer data)
 {
+	goffset blocks = 0;
 	gchar *image = NULL;
 	BraseroDvdcss *self = data;
 	BraseroDvdcssPrivate *priv;
@@ -171,6 +172,9 @@ brasero_dvdcss_thread_finished (gpointer data)
 					NULL,
 					BRASERO_IMAGE_FORMAT_BIN);
 
+	brasero_job_get_session_output_size (BRASERO_JOB (self), &blocks, NULL);
+	brasero_track_image_set_block_num (track, blocks);
+
 	brasero_job_add_track (BRASERO_JOB (self), BRASERO_TRACK (track));
 
 	/* It's good practice to unref the track afterwards as we don't need it
@@ -198,7 +202,7 @@ brasero_dvdcss_write_sector_to_fd (BraseroDvdcss *self,
 		gint written;
 
 		written = write (fd,
-				 buffer + bytes_written,
+				 ((gchar *) buffer)  + bytes_written,
 				 bytes_remaining);
 
 		if (priv->cancel)
@@ -366,7 +370,8 @@ brasero_dvdcss_write_image_thread (gpointer data)
 	brasero_volume_file_free (files);
 	files = NULL;
 
-	if (dvdcss_seek (handle, 0, DVDCSS_NOFLAGS) != 0) {
+	if (dvdcss_seek (handle, 0, DVDCSS_NOFLAGS) < 0) {
+		BRASERO_JOB_LOG (self, "Error initial seeking");
 		priv->error = g_error_new (BRASERO_BURN_ERROR,
 					   BRASERO_BURN_ERROR_GENERAL,
 					   _("Error while reading video DVD (%s)"),
@@ -389,14 +394,14 @@ brasero_dvdcss_write_image_thread (gpointer data)
 
 		brasero_job_get_image_output (BRASERO_JOB (self), &output, NULL);
 		output_fd = fopen (output, "w");
-		g_free (output);
-
 		if (!output_fd) {
 			priv->error = g_error_new_literal (BRASERO_BURN_ERROR,
 							   BRASERO_BURN_ERROR_GENERAL,
 							   g_strerror (errno));
+			g_free (output);
 			goto end;
 		}
+		g_free (output);
 	}
 
 	while (remaining_sectors) {
@@ -426,13 +431,18 @@ brasero_dvdcss_write_image_thread (gpointer data)
 			flag = DVDCSS_READ_DECRYPT;
 
 			/* see if we need to update the key */
-			if (written_sectors == range->start
-			&&  dvdcss_seek (handle, written_sectors, DVDCSS_SEEK_KEY) != written_sectors) {
-				priv->error = g_error_new (BRASERO_BURN_ERROR,
-							   BRASERO_BURN_ERROR_GENERAL,
-							   _("Error while reading video DVD (%s)"),
-							   dvdcss_error (handle));
-				break;
+			if (written_sectors == range->start) {
+				int pos;
+
+				pos = dvdcss_seek (handle, written_sectors, DVDCSS_SEEK_KEY);
+				if (pos < 0) {
+					BRASERO_JOB_LOG (self, "Error seeking");
+					priv->error = g_error_new (BRASERO_BURN_ERROR,
+								   BRASERO_BURN_ERROR_GENERAL,
+								   _("Error while reading video DVD (%s)"),
+								   dvdcss_error (handle));
+					break;
+				}
 			}
 
 			/* we don't want to mix scrambled and non scrambled sectors
@@ -447,7 +457,9 @@ brasero_dvdcss_write_image_thread (gpointer data)
 			}
 		}
 
-		if (dvdcss_read (handle, buf, num_blocks, flag) != num_blocks) {
+		num_blocks = dvdcss_read (handle, buf, num_blocks, flag);
+		if (num_blocks < 0) {
+			BRASERO_JOB_LOG (self, "Error reading");
 			priv->error = g_error_new (BRASERO_BURN_ERROR,
 						   BRASERO_BURN_ERROR_GENERAL,
 						   _("Error while reading video DVD (%s)"),
