@@ -449,7 +449,7 @@ brasero_libisofs_import_last_session (BraseroLibisofs *self,
 
 	priv = BRASERO_LIBISOFS_PRIVATE (self);
 
-	priv->ctx = brasero_libburn_common_ctx_new (BRASERO_JOB (self), error);
+	priv->ctx = brasero_libburn_common_ctx_new (BRASERO_JOB (self), FALSE, error);
 	if (!priv->ctx)
 		return BRASERO_BURN_ERR;
 
@@ -487,6 +487,7 @@ brasero_libisofs_import_last_session (BraseroLibisofs *self,
 	}
 
 	if (result < 0) {
+		BRASERO_JOB_LOG (self, "Import failed 0x%x", result);
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_IMAGE_LAST_SESSION,
@@ -548,6 +549,30 @@ brasero_libisofs_create_volume_thread (gpointer data)
 		goto end;
 	}
 
+	iso_write_opts_new (&opts, 2);
+
+	brasero_job_get_flags (BRASERO_JOB (self), &flags);
+	if (flags & BRASERO_BURN_FLAG_MERGE) {
+		BraseroBurnResult result;
+
+		result = brasero_libisofs_import_last_session (self,
+							       image,
+							       opts,
+							       &priv->error);
+		if (result != BRASERO_BURN_OK) {
+			g_free (label);
+			goto end;
+		}
+	}
+	else if (flags & BRASERO_BURN_FLAG_APPEND) {
+		goffset start_block;
+
+		brasero_job_get_next_writable_address (BRASERO_JOB (self), &start_block);
+		iso_write_opts_set_ms_block (opts, start_block);
+	}
+
+	/* set label but set it after merging so the
+	 * new does not get replaced by the former */
 	publisher = g_strdup_printf ("Brasero-%i.%i.%i",
 				     BRASERO_MAJOR_VERSION,
 				     BRASERO_MINOR_VERSION,
@@ -562,27 +587,6 @@ brasero_libisofs_create_volume_thread (gpointer data)
 	g_free (publisher);
 	g_free (label);
 
-	iso_write_opts_new (&opts, 2);
-
-	brasero_job_get_flags (BRASERO_JOB (self), &flags);
-	if (flags & BRASERO_BURN_FLAG_MERGE) {
-		BraseroBurnResult result;
-
-		result = brasero_libisofs_import_last_session (self,
-							       image,
-							       opts,
-							       &priv->error);
-		if (result != BRASERO_BURN_OK)
-			goto end;
-	}
-	else if (flags & BRASERO_BURN_FLAG_APPEND) {
-		goffset start_block;
-
-		brasero_job_get_next_writable_address (BRASERO_JOB (self), &start_block);
-		iso_write_opts_set_ms_block (opts, start_block);
-	}
-
-
 	brasero_job_start_progress (BRASERO_JOB (self), FALSE);
 
 	/* copy the list as we're going to reorder it */
@@ -876,6 +880,19 @@ brasero_libisofs_create_volume (BraseroLibisofs *self, GError **error)
 	return BRASERO_BURN_OK;
 }
 
+static void
+brasero_libisofs_clean_output (BraseroLibisofs *self)
+{
+	BraseroLibisofsPrivate *priv;
+
+	priv = BRASERO_LIBISOFS_PRIVATE (self);
+
+	if (priv->libburn_src) {
+		burn_source_free (priv->libburn_src);
+		priv->libburn_src = NULL;
+	}
+}
+
 static BraseroBurnResult
 brasero_libisofs_start (BraseroJob *job,
 			GError **error)
@@ -889,6 +906,10 @@ brasero_libisofs_start (BraseroJob *job,
 
 	brasero_job_get_action (job, &action);
 	if (action == BRASERO_JOB_ACTION_SIZE) {
+		/* do this to avoid a problem when using
+		 * DUMMY flag. libisofs would not generate
+		 * a second time. */
+		brasero_libisofs_clean_output (BRASERO_LIBISOFS (job));
 		brasero_job_set_current_action (BRASERO_JOB (self),
 						BRASERO_BURN_ACTION_GETTING_SIZE,
 						NULL,
@@ -977,22 +998,6 @@ brasero_libisofs_init (BraseroLibisofs *obj)
 	priv->cond = g_cond_new ();
 }
 
-static void
-brasero_libisofs_clean_output (BraseroLibisofs *self)
-{
-	BraseroLibisofsPrivate *priv;
-
-	priv = BRASERO_LIBISOFS_PRIVATE (self);
-
-	if (priv->libburn_src) {
-		burn_source_free (priv->libburn_src);
-		priv->libburn_src = NULL;
-	}
-
-	/* close libisofs library */
-	iso_finish ();
-}
-
 static void
 brasero_libisofs_finalize (GObject *object)
 {
@@ -1015,6 +1020,9 @@ brasero_libisofs_finalize (GObject *object)
 		priv->cond = NULL;
 	}
 
+	/* close libisofs library */
+	iso_finish ();
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
