@@ -175,6 +175,21 @@ brasero_file_chooser_customize (GtkWidget *widget, gpointer null_data)
 		}
 	}
 	else if (GTK_IS_CONTAINER (widget)) {
+		if (GTK_IS_PANED (widget)) {
+			GtkWidget *left;
+
+			/* This is to allow the left part to be shrunk as much 
+			 * as the user want. */
+			left = gtk_paned_get_child1 (GTK_PANED (widget));
+
+			g_object_ref (left);
+			gtk_container_remove (GTK_CONTAINER (widget), left);
+			gtk_paned_pack1 (GTK_PANED (widget),
+					 left,
+					 TRUE,
+					 TRUE);
+			g_object_unref (left);
+		}
 		gtk_container_foreach (GTK_CONTAINER (widget),
 				       brasero_file_chooser_customize,
 				       NULL);
@@ -214,7 +229,7 @@ brasero_file_chooser_init (BraseroFileChooser *obj)
 	obj->priv->filter_any = filter;
 
 	filter = gtk_file_filter_new ();
-	gtk_file_filter_set_name (filter, _("Audio files only"));
+	gtk_file_filter_set_name (filter, _("Audio files"));
 	gtk_file_filter_add_mime_type (filter, "audio/*");
 	gtk_file_filter_add_mime_type (filter, "application/ogg");
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (obj->priv->chooser), filter);
@@ -222,7 +237,7 @@ brasero_file_chooser_init (BraseroFileChooser *obj)
 	obj->priv->filter_audio = filter;
 
 	filter = gtk_file_filter_new ();
-	gtk_file_filter_set_name (filter, _("Movies only"));
+	gtk_file_filter_set_name (filter, _("Movies"));
 	gtk_file_filter_add_mime_type (filter, "video/*");
 	gtk_file_filter_add_mime_type (filter, "application/ogg");
 	gtk_file_filter_add_mime_type (filter, "application/x-flash-video");
@@ -231,7 +246,8 @@ brasero_file_chooser_init (BraseroFileChooser *obj)
 	obj->priv->filter_video = filter;
 
 	filter = gtk_file_filter_new ();
-	gtk_file_filter_set_name (filter, _("Image files only"));
+	/* Translators: this is an image, a picture, not a "Disc Image" */
+	gtk_file_filter_set_name (filter, C_("picture", "Image files"));
 	gtk_file_filter_add_mime_type (filter, "image/*");
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (obj->priv->chooser), filter);
 
