@@ -141,7 +141,54 @@ brasero_marshal_INT__STRING_BOOLEAN (GClosure     *closure,
   g_value_set_int (return_value, v_return);
 }
 
-/* INT:VOID (./brasero-marshal.list:3) */
+/* INT:POINTER,BOOLEAN (./brasero-marshal.list:3) */
+extern void brasero_marshal_INT__POINTER_BOOLEAN (GClosure     *closure,
+                                                  GValue       *return_value,
+                                                  guint         n_param_values,
+                                                  const GValue *param_values,
+                                                  gpointer      invocation_hint,
+                                                  gpointer      marshal_data);
+void
+brasero_marshal_INT__POINTER_BOOLEAN (GClosure     *closure,
+                                      GValue       *return_value G_GNUC_UNUSED,
+                                      guint         n_param_values,
+                                      const GValue *param_values,
+                                      gpointer      invocation_hint G_GNUC_UNUSED,
+                                      gpointer      marshal_data)
+{
+  typedef gint (*GMarshalFunc_INT__POINTER_BOOLEAN) (gpointer     data1,
+                                                     gpointer     arg_1,
+                                                     gboolean     arg_2,
+                                                     gpointer     data2);
+  register GMarshalFunc_INT__POINTER_BOOLEAN callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gint v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_INT__POINTER_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_pointer (param_values + 1),
+                       g_marshal_value_peek_boolean (param_values + 2),
+                       data2);
+
+  g_value_set_int (return_value, v_return);
+}
+
+/* INT:VOID (./brasero-marshal.list:4) */
 extern void brasero_marshal_INT__VOID (GClosure     *closure,
                                        GValue       *return_value,
                                        guint         n_param_values,
@@ -184,7 +231,7 @@ brasero_marshal_INT__VOID (GClosure     *closure,
   g_value_set_int (return_value, v_return);
 }
 
-/* INT:INT (./brasero-marshal.list:4) */
+/* INT:INT (./brasero-marshal.list:5) */
 extern void brasero_marshal_INT__INT (GClosure     *closure,
                                       GValue       *return_value,
                                       guint         n_param_values,
@@ -229,7 +276,7 @@ brasero_marshal_INT__INT (GClosure     *closure,
   g_value_set_int (return_value, v_return);
 }
 
-/* INT:INT,INT (./brasero-marshal.list:5) */
+/* INT:INT,INT (./brasero-marshal.list:6) */
 extern void brasero_marshal_INT__INT_INT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -276,7 +323,7 @@ brasero_marshal_INT__INT_INT (GClosure     *closure,
   g_value_set_int (return_value, v_return);
 }
 
-/* INT:STRING (./brasero-marshal.list:6) */
+/* INT:STRING (./brasero-marshal.list:7) */
 extern void brasero_marshal_INT__STRING (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -321,7 +368,7 @@ brasero_marshal_INT__STRING (GClosure     *closure,
   g_value_set_int (return_value, v_return);
 }
 
-/* INT:BOOLEAN,BOOLEAN,BOOLEAN (./brasero-marshal.list:7) */
+/* INT:BOOLEAN,BOOLEAN,BOOLEAN (./brasero-marshal.list:8) */
 extern void brasero_marshal_INT__BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                           GValue       *return_value,
                                                           guint         n_param_values,
@@ -370,7 +417,7 @@ brasero_marshal_INT__BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
   g_value_set_int (return_value, v_return);
 }
 
-/* VOID:DOUBLE,STRING (./brasero-marshal.list:8) */
+/* VOID:DOUBLE,STRING (./brasero-marshal.list:9) */
 extern void brasero_marshal_VOID__DOUBLE_STRING (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -413,7 +460,7 @@ brasero_marshal_VOID__DOUBLE_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT64 (./brasero-marshal.list:9) */
+/* VOID:INT64 (./brasero-marshal.list:10) */
 extern void brasero_marshal_VOID__INT64 (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -454,7 +501,7 @@ brasero_marshal_VOID__INT64 (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,STRING (./brasero-marshal.list:10) */
+/* VOID:INT,STRING (./brasero-marshal.list:11) */
 extern void brasero_marshal_VOID__INT_STRING (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -497,7 +544,7 @@ brasero_marshal_VOID__INT_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,STRING (./brasero-marshal.list:11) */
+/* VOID:POINTER,STRING (./brasero-marshal.list:12) */
 extern void brasero_marshal_VOID__POINTER_STRING (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -540,7 +587,7 @@ brasero_marshal_VOID__POINTER_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:DOUBLE,DOUBLE,LONG (./brasero-marshal.list:12) */
+/* VOID:DOUBLE,DOUBLE,LONG (./brasero-marshal.list:13) */
 extern void brasero_marshal_VOID__DOUBLE_DOUBLE_LONG (GClosure     *closure,
                                                       GValue       *return_value,
                                                       guint         n_param_values,
@@ -585,7 +632,7 @@ brasero_marshal_VOID__DOUBLE_DOUBLE_LONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING (./brasero-marshal.list:13) */
+/* VOID:STRING,STRING (./brasero-marshal.list:14) */
 extern void brasero_marshal_VOID__STRING_STRING (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -628,7 +675,7 @@ brasero_marshal_VOID__STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* INT:OBJECT,INT,INT (./brasero-marshal.list:14) */
+/* INT:OBJECT,INT,INT (./brasero-marshal.list:15) */
 extern void brasero_marshal_INT__OBJECT_INT_INT (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -677,7 +724,7 @@ brasero_marshal_INT__OBJECT_INT_INT (GClosure     *closure,
   g_value_set_int (return_value, v_return);
 }
 
-/* BOOLEAN:STRING (./brasero-marshal.list:15) */
+/* BOOLEAN:STRING (./brasero-marshal.list:16) */
 extern void brasero_marshal_BOOLEAN__STRING (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -722,7 +769,7 @@ brasero_marshal_BOOLEAN__STRING (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* VOID:OBJECT,BOOLEAN (./brasero-marshal.list:16) */
+/* VOID:OBJECT,BOOLEAN (./brasero-marshal.list:17) */
 extern void brasero_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -765,7 +812,7 @@ brasero_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOOLEAN,BOOLEAN (./brasero-marshal.list:17) */
+/* VOID:BOOLEAN,BOOLEAN (./brasero-marshal.list:18) */
 extern void brasero_marshal_VOID__BOOLEAN_BOOLEAN (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
