@@ -36,6 +36,11 @@
 
 #include <gtk/gtk.h>
 
+#include "brasero-io.h"
+
+#include "brasero-units.h"
+#include "brasero-medium-monitor.h"
+
 #include "brasero-app.h"
 #include "brasero-utils.h"
 #include "brasero-project.h"
@@ -44,8 +49,6 @@
 #include "brasero-file-chooser.h"
 #include "brasero-uri-container.h"
 #include "brasero-project-type-chooser.h"
-#include "brasero-io.h"
-#include "brasero-medium-monitor.h"
 
 #ifdef BUILD_SEARCH
 #include "brasero-search-beagle.h"
@@ -214,7 +217,7 @@ brasero_project_manager_new_cover_cb (GtkAction *action,
 	GtkWidget *dialog;
 
 	if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO) {
-		brasero_project_create_audio_cover (BRASERO_PROJECT (manager->priv->project), edit);
+		brasero_project_create_audio_cover (BRASERO_PROJECT (manager->priv->project));
 		return;
 	}
 
@@ -246,7 +249,8 @@ brasero_project_manager_set_statusbar (BraseroProjectManager *manager,
 	if (!invalid_num && valid_num) {
 		gchar *size_string;
 
-		if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO)
+		if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO
+		||  manager->priv->type == BRASERO_PROJECT_TYPE_VIDEO)
 			size_string = brasero_units_get_time_string (files_size, TRUE, FALSE);
 		else if (manager->priv->type == BRASERO_PROJECT_TYPE_DATA)
 			size_string = g_format_size_for_display (files_size);
@@ -261,7 +265,8 @@ brasero_project_manager_set_statusbar (BraseroProjectManager *manager,
 	else if (valid_num) {
 		gchar *size_string = NULL;
 
-		if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO) {
+		if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO
+		||  manager->priv->type == BRASERO_PROJECT_TYPE_VIDEO) {
 			size_string = brasero_units_get_time_string (files_size, TRUE, FALSE);
 			status_string = g_strdup_printf (ngettext ("%d file is supported (%s)", "%d files are supported (%s)", valid_num),
 							 valid_num,
@@ -746,175 +751,34 @@ brasero_project_manager_iso (BraseroProjectManager *manager,
 					TRUE);
 }
 
-BraseroProjectType
-brasero_project_manager_open_project (BraseroProjectManager *manager,
-				      BraseroDiscTrack *track,
-				      const gchar *uri,
-				      gboolean burn)
+gboolean
+brasero_project_manager_open_session (BraseroProjectManager *manager,
+                                      BraseroSessionCfg *session,
+                                      gboolean burn)
 {
 	GtkAction *action;
 	BraseroProjectType type;
 
-    	if (track->type == BRASERO_PROJECT_TYPE_INVALID) {
+	type = brasero_project_open_session (BRASERO_PROJECT (manager->priv->project), session);
+	if (type == BRASERO_PROJECT_TYPE_INVALID) {
 		brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
-		return BRASERO_PROJECT_TYPE_INVALID;
+		return FALSE;
 	}
 
-	brasero_project_manager_switch (manager, track->type, NULL, NULL, FALSE);
-	type = brasero_project_open_project (BRASERO_PROJECT (manager->priv->project), track, uri);
-	if (type == BRASERO_PROJECT_TYPE_INVALID)
-		return type;
+	brasero_project_manager_switch (manager,
+					type,
+					NULL,
+					NULL,
+					FALSE);
 
 	if (burn) {
 		brasero_project_burn (BRASERO_PROJECT (manager->priv->project));
-		return track->type;
+		return TRUE;
 	}
 
 	action = gtk_action_group_get_action (manager->priv->action_group, "NewChoose");
 	gtk_action_set_sensitive (action, TRUE);
-
-	return track->type;
-}
-
-BraseroProjectType
-brasero_project_manager_open_by_mime (BraseroProjectManager *manager,
-				      const gchar *uri,
-				      const gchar *mime)
-{
-	/* When our files/description of x-brasero mime type is not properly 
-	 * installed, it's returned as application/xml, so check that too. */
-	if (!strcmp (mime, "application/x-brasero")
-	||  !strcmp (mime, "application/xml")) {
-		BraseroDiscTrack *track = NULL;
-
-		if (!brasero_project_open_project_xml (uri, &track, TRUE))
-			return BRASERO_PROJECT_TYPE_INVALID;
-
-		return brasero_project_manager_open_project (manager, track, uri, FALSE);
-	}
-
-#ifdef BUILD_PLAYLIST
-
-	else if (!strcmp (mime, "audio/x-scpls")
-	     ||  !strcmp (mime, "audio/x-ms-asx")
-	     ||  !strcmp (mime, "audio/x-mp3-playlist")
-	     ||  !strcmp (mime, "audio/x-mpegurl")) {
-		BraseroDiscTrack *track = NULL;
-
-		if (!brasero_project_open_audio_playlist_project (uri, &track, TRUE))
-			return BRASERO_PROJECT_TYPE_INVALID;
-
-		return brasero_project_manager_open_project (manager, track, uri, FALSE);
-	}
-#endif
-
-	else if (!strcmp (mime, "application/x-cd-image")
-	     ||  !strcmp (mime, "application/x-cdrdao-toc")
-	     ||  !strcmp (mime, "application/x-toc")
-	     ||  !strcmp (mime, "application/x-cue")) {
-		brasero_project_manager_iso (manager, uri);
-		return BRASERO_PROJECT_TYPE_ISO;
-	}
-
-	return BRASERO_PROJECT_TYPE_INVALID;
-}
-
-BraseroProjectType
-brasero_project_manager_open_uri (BraseroProjectManager *manager,
-				  const gchar *uri_arg)
-{
-	gchar *uri;
-	GFile *file;
-	GFileInfo *info;
-	BraseroProjectType type;
-
-	/* FIXME: make that asynchronous */
-	/* NOTE: don't follow symlink because we want to identify them */
-	file = g_file_new_for_commandline_arg (uri_arg);
-	if (!file)
-		return BRASERO_PROJECT_TYPE_INVALID;
-
-	info = g_file_query_info (file,
-				  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
-				  G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK ","
-				  G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET,
-				  G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-				  NULL,
-				  NULL);
-
-	if (!info) {
-		g_object_unref (file);
-		return BRASERO_PROJECT_TYPE_INVALID;
-	}
-
-	/* if that's a symlink, redo it on its target to get the real mime type
-	 * that usually also depends on the extension of the target:
-	 * ex: an iso file with the extension .iso will be seen as octet-stream
-	 * if the symlink hasn't got any extention at all */
-	while (g_file_info_get_is_symlink (info)) {
-		const gchar *target;
-		GFileInfo *tmp_info;
-		GFile *tmp_file;
-		GError *error = NULL;
-
-		target = g_file_info_get_symlink_target (info);
-		if (!g_path_is_absolute (target)) {
-			gchar *parent;
-			gchar *tmp;
-
-			tmp = g_file_get_path (file);
-			parent = g_path_get_dirname (tmp);
-			g_free (tmp);
-
-			target = g_build_filename (parent, target, NULL);
-			g_free (parent);
-		}
-
-		tmp_file = g_file_new_for_commandline_arg (target);
-		tmp_info = g_file_query_info (tmp_file,
-					      G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
-					      G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK ","
-					      G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET,
-					      G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-					      NULL,
-					      &error);
-		if (!tmp_info) {
-			g_object_unref (tmp_file);
-			break;
-		}
-
-		g_object_unref (info);
-		g_object_unref (file);
-
-		info = tmp_info;
-		file = tmp_file;
-	}
-
-	uri = g_file_get_uri (file);
-	if (g_file_query_exists (file, NULL)) {
-		const gchar *mime;
-
-		mime = g_file_info_get_content_type (info);
-	  	type = brasero_project_manager_open_by_mime (manager, uri, mime);
-        } 
-	else {
-		gchar *string;
-
-		string = g_strdup_printf (_("The project \"%s\" does not exist"), uri);
-		brasero_app_alert (brasero_app_get_default (),
-				   _("Error while loading the project"),
-				   string,
-				   GTK_MESSAGE_ERROR);
-		g_free (string);
-
-		type = BRASERO_PROJECT_TYPE_INVALID;
-	}
-
-	g_free (uri);
-	g_object_unref (file);
-	g_object_unref (info);
-
-	return type;
+	return TRUE;
 }
 
 static void
@@ -946,7 +810,7 @@ brasero_project_manager_open_cb (GtkAction *action, BraseroProjectManager *manag
 	uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (chooser));
 	gtk_widget_destroy (chooser);
 
-	brasero_project_manager_open_uri (manager, uri);
+	brasero_app_open_uri (brasero_app_get_default (), uri, TRUE);
 	g_free (uri);
 }
 
@@ -955,7 +819,7 @@ brasero_project_manager_recent_clicked_cb (BraseroProjectTypeChooser *chooser,
 					   const gchar *uri,
 					   BraseroProjectManager *manager)
 {
-	brasero_project_manager_open_uri (manager, uri);
+	brasero_app_open_uri (brasero_app_get_default (), uri, TRUE);
 }
 
 void
@@ -964,30 +828,22 @@ brasero_project_manager_empty (BraseroProjectManager *manager)
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
 }
 
-gboolean
-brasero_project_manager_load_session (BraseroProjectManager *manager,
-				      const gchar *path)
-{
-    	if (path) {
-		gchar *uri;
-		BraseroProjectType type;
-
-		uri = g_filename_to_uri (path, NULL, NULL);
-    		type = brasero_project_load_session (BRASERO_PROJECT (manager->priv->project), uri);
-		g_free (uri);
-
-		brasero_project_manager_switch (manager, type, NULL, NULL, FALSE);
-	}
-
-    	return TRUE;
-}
-
 static void
 brasero_project_manager_last_saved_clicked_cb (BraseroProjectTypeChooser *chooser,
 					       const gchar *path,
 					       BraseroProjectManager *manager)
 {
-	brasero_project_manager_load_session (manager, path);
+	if (path) {
+		gchar *uri;
+
+		uri = g_filename_to_uri (path, NULL, NULL);
+		brasero_app_open_project (brasero_app_get_default (),
+		                          uri,
+		                          FALSE, // not a playlist
+		                          FALSE, // should work so don't warn user
+		                          FALSE); // don't burn right away
+		g_free (uri);
+	}
 }
 
 gboolean
