@@ -368,7 +368,7 @@ brasero_project_set_add_button_state (BraseroProject *project)
 	GtkWidget *toplevel;
 
 	sensitive = ((!project->priv->current_source || !project->priv->has_focus) &&
-		      !project->priv->oversized);
+		      !project->priv->oversized && !project->priv->chooser);
 
 	action = gtk_action_group_get_action (project->priv->project_group, "Add");
 	gtk_action_set_sensitive (action, sensitive);
@@ -943,6 +943,7 @@ brasero_project_is_valid (BraseroSessionCfg *session,
 
 	if (valid == BRASERO_SESSION_EMPTY) {
 		project->priv->empty = TRUE;
+		project->priv->oversized = FALSE;
 	}
 	else if (valid == BRASERO_SESSION_INSUFFICIENT_SPACE) {
 		goffset min_disc_size;
@@ -1606,12 +1607,16 @@ brasero_project_reset (BraseroProject *project)
 }
 
 static void
-brasero_project_new_session (BraseroProject *project)
+brasero_project_new_session (BraseroProject *project,
+                             BraseroSessionCfg *session)
 {
 	if (project->priv->session)
 		brasero_project_reset (project);
 
-	project->priv->session = brasero_session_cfg_new ();
+	if (session)
+		project->priv->session = g_object_ref (session);
+	else
+		project->priv->session = brasero_session_cfg_new ();
 
 	/* NOTE: "is-valid" is emitted whenever there is a change in the
 	 * contents of the session. So no need to connect to track-added, ... */
@@ -1647,7 +1652,8 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 		project->priv->current = BRASERO_DISC (project->priv->audio);
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (project->priv->discs), 0);
 		brasero_medium_selection_show_media_type (BRASERO_MEDIUM_SELECTION (project->priv->selection),
-							  BRASERO_MEDIA_TYPE_WRITABLE);
+							  BRASERO_MEDIA_TYPE_WRITABLE|
+		                                          BRASERO_MEDIA_TYPE_CD);
 	}
 	else if (type == BRASERO_PROJECT_TYPE_DATA) {
 		project->priv->current = BRASERO_DISC (project->priv->data);
@@ -1695,7 +1701,7 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 void
 brasero_project_set_audio (BraseroProject *project, GSList *uris)
 {
-	brasero_project_new_session (project);
+	brasero_project_new_session (project, NULL);
 	brasero_project_switch (project, BRASERO_PROJECT_TYPE_AUDIO);
 
 	for (; uris; uris = uris->next) {
@@ -1710,7 +1716,7 @@ void
 brasero_project_set_data (BraseroProject *project,
 			  GSList *uris)
 {
-	brasero_project_new_session (project);
+	brasero_project_new_session (project, NULL);
 	brasero_project_switch (project, BRASERO_PROJECT_TYPE_DATA);
 
 	for (; uris; uris = uris->next) {
@@ -1724,7 +1730,7 @@ brasero_project_set_data (BraseroProject *project,
 void
 brasero_project_set_video (BraseroProject *project, GSList *uris)
 {
-	brasero_project_new_session (project);
+	brasero_project_new_session (project, NULL);
 	brasero_project_switch (project, BRASERO_PROJECT_TYPE_VIDEO);
 
 	for (; uris; uris = uris->next) {
@@ -1899,10 +1905,24 @@ static void
 brasero_project_file_chooser_activated_cb (GtkWidget *chooser,
 					   BraseroProject *project)
 {
+	gboolean sensitive;
+	GtkAction *action;
 	GSList *uris;
 	GSList *iter;
 
+	if (!project->priv->chooser)
+		return;
+
+	project->priv->chooser = NULL;
 	uris = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (chooser));
+	gtk_widget_destroy (GTK_WIDGET (chooser));
+
+	sensitive = ((!project->priv->current_source || !project->priv->has_focus) &&
+		      !project->priv->oversized);
+
+	action = gtk_action_group_get_action (project->priv->project_group, "Add");
+	gtk_action_set_sensitive (action, sensitive);
+
 	for (iter = uris; iter; iter = iter->next) {
 		gchar *uri;
 
@@ -1911,9 +1931,6 @@ brasero_project_file_chooser_activated_cb (GtkWidget *chooser,
 	}
 	g_slist_foreach (uris, (GFunc) g_free, NULL);
 	g_slist_free (uris);
-
-	gtk_widget_destroy (GTK_WIDGET (project->priv->chooser));
-	project->priv->chooser = NULL;
 }
 
 static void
@@ -1921,16 +1938,30 @@ brasero_project_file_chooser_response_cb (GtkWidget *chooser,
 					  GtkResponseType response,
 					  BraseroProject *project)
 {
+	gboolean sensitive;
+	GtkAction *action;
 	GSList *uris;
 	GSList *iter;
 
+	if (!project->priv->chooser)
+		return;
+
 	if (response != BRASERO_RESPONSE_ADD) {
 		gtk_widget_destroy (chooser);
 		project->priv->chooser = NULL;
 		return;
 	}
 
+	project->priv->chooser = NULL;
 	uris = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (chooser));
+	gtk_widget_destroy (GTK_WIDGET (chooser));
+
+	sensitive = ((!project->priv->current_source || !project->priv->has_focus) &&
+		      !project->priv->oversized);
+
+	action = gtk_action_group_get_action (project->priv->project_group, "Add");
+	gtk_action_set_sensitive (action, sensitive);
+
 	for (iter = uris; iter; iter = iter->next) {
 		gchar *uri;
 
@@ -1939,9 +1970,6 @@ brasero_project_file_chooser_response_cb (GtkWidget *chooser,
 	}
 	g_slist_foreach (uris, (GFunc) g_free, NULL);
 	g_slist_free (uris);
-
-	gtk_widget_destroy (GTK_WIDGET (project->priv->chooser));
-	project->priv->chooser = NULL;
 }
 
 static void
@@ -1976,6 +2004,11 @@ brasero_project_add_uris_cb (GtkAction *action,
 		return;
 	}
 
+	/* set the Add button grey as we don't want
+	 * the user to be able to click again until it
+	 * the dialog has been closed */
+	gtk_action_set_sensitive (action, FALSE);
+
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (project));
 	project->priv->chooser = gtk_file_chooser_dialog_new (_("Select Files"),
 							      GTK_WINDOW (toplevel),
@@ -1983,7 +2016,7 @@ brasero_project_add_uris_cb (GtkAction *action,
 							      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 							      NULL);
 	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (project->priv->chooser), TRUE);
-	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (project->priv->chooser), TRUE);
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (project->priv->chooser), FALSE);
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (project->priv->chooser), g_get_home_dir ());
 	brasero_file_chooser_customize (project->priv->chooser, NULL);
 	gtk_widget_show (project->priv->chooser);
@@ -2281,30 +2314,15 @@ brasero_project_get_session_type (BraseroProject *project)
 
 /******************************* Projects **************************************/
 BraseroProjectType
-brasero_project_open_project_real (BraseroProject *project,
-				   const gchar *uri,	/* escaped */
-				   gboolean playlist,
-				   gboolean warn_user)
+brasero_project_open_session (BraseroProject *project,
+                              BraseroSessionCfg *session)
 {
 	GValue *value;
 	BraseroProjectType type;
 
-	brasero_project_new_session (project);
+	brasero_project_new_session (project, session);
 
-#ifdef BUILD_PLAYLIST
-
-	if (playlist) {
-		if (!brasero_project_open_audio_playlist_project (uri, BRASERO_BURN_SESSION (project->priv->session), warn_user))
-			return BRASERO_PROJECT_TYPE_INVALID;
-	}
-	else
-
-#endif
-	
-	if (!brasero_project_open_project_xml (uri, BRASERO_BURN_SESSION (project->priv->session), warn_user))
-		return BRASERO_PROJECT_TYPE_INVALID;
-
-        type = brasero_project_get_session_type (project);
+	type = brasero_project_get_session_type (project);
         if (type == BRASERO_PROJECT_TYPE_INVALID)
                 return type;
 
@@ -2337,27 +2355,6 @@ brasero_project_open_project_real (BraseroProject *project,
 	return type;
 }
 
-BraseroProjectType
-brasero_project_open_project (BraseroProject *project,
-			      const gchar *uri,
-			      gboolean playlist)	/* escaped */
-{
-	BraseroProjectType type;
-
-	type = brasero_project_open_project_real (project, uri, playlist, TRUE);
-	if (type != BRASERO_PROJECT_TYPE_INVALID && uri)
-		brasero_project_set_uri (project, uri, type);
-
-	return type;
-}
-
-BraseroProjectType
-brasero_project_load_session (BraseroProject *project,
-			      const gchar *uri)
-{
-	return brasero_project_open_project_real (project, uri, FALSE, FALSE);
-}
-
 /******************************** save project *********************************/
 static void
 brasero_project_not_saved_dialog (BraseroProject *project)
@@ -2674,10 +2671,32 @@ brasero_project_save_session (BraseroProject *project,
 		return FALSE;
 	}
 
-	/* NOTE: is this right? because brasero could not shut itself
-	 * down if an error occurs. */
-	if (!brasero_project_save_project_xml (BRASERO_BURN_SESSION (project->priv->session), uri))
-		return TRUE;
+	if (!brasero_project_save_project_xml (BRASERO_BURN_SESSION (project->priv->session), uri)) {
+		GtkResponseType response;
+		GtkWidget *dialog;
+
+		/* If the automatic saving failed, let the user decide */
+		dialog = brasero_app_dialog (brasero_app_get_default (),
+					      _("Your project has not been saved."),
+					     GTK_BUTTONS_NONE,
+					     GTK_MESSAGE_WARNING);
+
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+							  _("If you don't save, changes will be permanently lost."));
+
+		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+		                        _("Cl_ose Without Saving"), GTK_RESPONSE_NO,
+		                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		                        NULL);
+
+		response = gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+
+		if (saved_uri)
+			*saved_uri = NULL;
+
+		return (response == GTK_RESPONSE_CANCEL);
+	}
 
 	if (saved_uri)
 		*saved_uri = g_strdup (uri);
