@@ -459,7 +459,7 @@ brasero_video_disc_session_changed (BraseroSessionCfg *session,
 			error = brasero_status_get_error (status);
 			if (!error)
 				brasero_video_disc_unreadable_uri_dialog (self, uri, error);
-			else if (error->code != BRASERO_BURN_ERROR_FILE_FOLDER) {
+			else if (error->code == BRASERO_BURN_ERROR_FILE_FOLDER) {
 				res = brasero_video_disc_directory_dialog (self);
 				if (res)
 					brasero_video_disc_add_directory_contents (self, uri, BRASERO_TRACK (track));
@@ -1058,7 +1058,7 @@ brasero_video_disc_add_ui (BraseroDisc *disc,
 	g_object_ref (message);
 
 	if (!priv->disc_group) {
-		priv->disc_group = gtk_action_group_new (BRASERO_DISC_ACTION);
+		priv->disc_group = gtk_action_group_new (BRASERO_DISC_ACTION "-video");
 		gtk_action_group_set_translation_domain (priv->disc_group, GETTEXT_PACKAGE);
 		gtk_action_group_add_actions (priv->disc_group,
 					      entries,
@@ -1324,20 +1324,26 @@ brasero_video_disc_set_session_contents (BraseroDisc *self,
 					 BraseroBurnSession *session)
 {
 	BraseroVideoTreeModel *model;
+	GtkTreeModel *current_model;
 	BraseroVideoDiscPrivate *priv;
 
 	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
 
+	if (priv->load_dir)
+		brasero_io_cancel_by_base (priv->load_dir);
+
+	current_model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
+	if (current_model) {
+		BraseroSessionCfg *current_session;
+
+		current_session = brasero_video_tree_model_get_session (BRASERO_VIDEO_TREE_MODEL (current_model));
+		if (current_session)
+			g_signal_handlers_disconnect_by_func (current_session,
+							      brasero_video_disc_session_changed,
+							      self);
+	}
+
 	if (!session) {
-		GtkTreeModel *model;
-		BraseroSessionCfg *session;
-
-		model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
-		session = brasero_video_tree_model_get_session (BRASERO_VIDEO_TREE_MODEL (model));
-		g_signal_handlers_disconnect_by_func (session,
-						      brasero_video_disc_session_changed,
-						      self);
-
 		gtk_tree_view_set_model (GTK_TREE_VIEW (priv->tree), NULL);
 		return BRASERO_DISC_OK;
 	}
